"""
Fused Attention
===============

This is a Triton implementation of the Flash Attention v2 algorithm from Tri Dao (https://tridao.me/publications/flash2/flash2.pdf)

Extra Credits:
- Original flash attention paper (https://arxiv.org/abs/2205.14135)
- Rabe and Staats (https://arxiv.org/pdf/2112.05682v2.pdf)
- Adam P. Goucher for simplified vector math

This version was modified to fuse an addition of two attention masks into one
attn_bias = (rel_h_ + rel_w_).view(q_.size(0), q_.size(1), rel_h_.size(2), rel_h_.size(3) * rel_w_.size(4))

We use attn_mask and attn_bias interchangeably.

This modification was designed by Christian Puhrsch and Daniel Haziza

"""

import torch

import triton
import triton.language as tl

import os
import pathlib


@triton.jit
def _fwd_kernel_aligned(
    Q, K, V, B0, sm_scale,
    Out,
    stride_qh, stride_qm, stride_qk,
    stride_kh, stride_kn, stride_kk,
    stride_vh, stride_vk, stride_vn,
    stride_oh, stride_om, stride_on,
    stride_b0h, stride_b0m,
    Z,
    H,
    N_CTX,
    P_SEQ,
    OUT_DTYPE: tl.constexpr,
    BIAS_LAST_SIZE: tl.constexpr,
    B0_NUMEL: tl.constexpr,
    BLOCK_DMODEL: tl.constexpr,
    BLOCK_M: tl.constexpr,
    BLOCK_N: tl.constexpr,
):
    start_m = tl.program_id(0)
    off_hz = tl.program_id(1)
    q_offset = off_hz * stride_qh
    kv_offset = off_hz * stride_kh
    Q_block_ptr = tl.make_block_ptr(
        base=Q + q_offset,
        shape=(N_CTX, BLOCK_DMODEL),
        strides=(stride_qm, stride_qk),
        offsets=(start_m * BLOCK_M, 0),
        block_shape=(BLOCK_M, BLOCK_DMODEL),
        order=(1, 0)
    )
    K_block_ptr = tl.make_block_ptr(
        base=K + kv_offset,
        shape=(BLOCK_DMODEL, N_CTX + P_SEQ),
        strides=(stride_kk, stride_kn),
        offsets=(0, 0),
        block_shape=(BLOCK_DMODEL, BLOCK_N),
        order=(0, 1)
    )
    V_block_ptr = tl.make_block_ptr(
        base=V + kv_offset,
        shape=(N_CTX + P_SEQ, BLOCK_DMODEL),
        strides=(stride_vk, stride_vn),
        offsets=(0, 0),
        block_shape=(BLOCK_N, BLOCK_DMODEL),
        order=(1, 0)
    )

    # initialize offsets
    # initialize pointer to m and l
    m_i = tl.zeros([BLOCK_M], dtype=tl.float32) - float("inf")
    l_i = tl.zeros([BLOCK_M], dtype=tl.float32)
    acc = tl.zeros([BLOCK_M, BLOCK_DMODEL], dtype=tl.float32)
    # scale sm_scale by log_2(e) and use
    # 2^x instead of exp in the loop because CSE and LICM
    # don't work as expected with `exp` in the loop
    qk_scale = sm_scale * 1.44269504
    # load q: it will stay in SRAM throughout
    q = tl.load(Q_block_ptr)  # , boundary_check=(1, 0), padding_option="zero")
    q = (q * qk_scale).to(OUT_DTYPE)
    # loop over k, v and update accumulator
    lo = 0
    hi = N_CTX + P_SEQ

    b_ptr_offsets_m = tl.arange(0, BLOCK_M)

    b_offset = off_hz * stride_b0h
    b_ptr_offsets_n_1 = (tl.arange(0, BLOCK_N) %
                         BIAS_LAST_SIZE) + BIAS_LAST_SIZE
    b1 = tl.load(B0 + b_offset + ((start_m * BLOCK_M + b_ptr_offsets_m)
                 * stride_b0m)[:, None] + b_ptr_offsets_n_1[None, :])
    for start_n in range(lo, hi, BLOCK_N):
        # -- load k, v --
        # , boundary_check=(0, 1), padding_option="zero")
        k = tl.load(K_block_ptr)
        # , boundary_check=(1, 0), padding_option="zero")
        v = tl.load(V_block_ptr)
        # -- compute qk ---
        qk = tl.zeros([BLOCK_M, BLOCK_N], dtype=OUT_DTYPE)
        qk += tl.dot(q, k, out_dtype=OUT_DTYPE)

        # -- compute rel_h[:, None] + rel_w[None, :] bias ---

        # Bias
        b0 = tl.load(B0 + b_offset + ((start_m * BLOCK_M + b_ptr_offsets_m)
                     * stride_b0m)[:, None] + start_n // BLOCK_N)
        qk += (b0 + b1)

        # -- compute scaling constant ---
        m_i_new = tl.maximum(m_i, tl.max(qk, 1))
        alpha = tl.math.exp2(m_i - m_i_new)
        p = tl.math.exp2(qk - m_i_new[:, None])
        # -- scale and update acc --
        acc *= alpha[:, None]
        acc += tl.dot(p.to(OUT_DTYPE), v)
        # -- update m_i and l_i --
        l_i = l_i * alpha + tl.sum(p, 1)
        m_i = m_i_new
        # update pointers
        K_block_ptr = tl.advance(K_block_ptr, (0, BLOCK_N))
        V_block_ptr = tl.advance(V_block_ptr, (BLOCK_N, 0))

    # write back l and m
    acc = acc / l_i[:, None]

    # write back O
    O_block_ptr = tl.make_block_ptr(
        base=Out + q_offset,
        shape=(N_CTX, BLOCK_DMODEL),
        strides=(stride_om, stride_on),
        offsets=(start_m * BLOCK_M, 0),
        block_shape=(BLOCK_M, BLOCK_DMODEL),
        order=(1, 0)
    )
    tl.store(O_block_ptr, acc.to(OUT_DTYPE))


def _autotune(configs, function):
    import torch.utils.benchmark as benchmark

    def benchmark_torch_function_in_microseconds(f, *args, **kwargs):
        try:
            f(*args, **kwargs)
            t0 = benchmark.Timer(
                stmt="f(*args, **kwargs)", globals={"args": args, "kwargs": kwargs, "f": f}
            )
        except:
            return None
        return t0.blocked_autorange().mean * 1e6

    best = None
    best_config = None
    for config in configs:
        BLOCK_M, BLOCK_N, num_warps, num_stages = config
        t_config = benchmark_torch_function_in_microseconds(
            function, BLOCK_M, BLOCK_N, num_warps, num_stages)
        if t_config is not None:
            if best is not None:
                if t_config < best:
                    best = t_config
                    best_config = config
            else:
                best = t_config
                best_config = config
        print(str(config), " :", str(t_config))
    return best, best_config


def _attention_rel_h_rel_w_kernel_aligned_device(q, k, v, rel_h_w, sm_scale, o,
                                                 BLOCK_M,
                                                 BLOCK_N,
                                                 num_warps,
                                                 num_stages):
    _, Lk, _ = q.shape[-1], k.shape[-1], v.shape[-1]
    assert q.size() == k.size()
    assert q.size() == v.size()
    assert q.size(-2) == rel_h_w.size(-2)
    assert (q.dtype == torch.bfloat16 or q.dtype == torch.float16)
    assert k.dtype == q.dtype
    assert v.dtype == k.dtype
    assert o.dtype == v.dtype
    assert rel_h_w.dtype == q.dtype
    assert rel_h_w.size(-1) == 128
    # assert rel_h_w.size(-1) == 2 * BLOCK_N

    grid = (triton.cdiv(q.shape[2], BLOCK_M), q.shape[0] * q.shape[1], 1)
    # print("q.shape[0] * q.shape[1]: ", q.shape[0] * q.shape[1])
    P_SEQ = 0 if q.shape[-2] == k.shape[-2] else k.shape[-2] - q.shape[-2]
    assert P_SEQ == 0
    assert rel_h_w.is_contiguous(), str(rel_h_w.stride())
    _fwd_kernel_aligned[grid](
        q, k, v,
        rel_h_w,
        sm_scale,
        o,
        q.stride(1), q.stride(2), q.stride(3),
        k.stride(1), k.stride(2), k.stride(3),
        v.stride(1), v.stride(2), v.stride(3),
        o.stride(1), o.stride(2), o.stride(3),
        rel_h_w.stride(1), rel_h_w.stride(2),
        q.shape[0],
        q.shape[1],
        q.shape[2],
        P_SEQ,
        OUT_DTYPE=tl.float16 if q.dtype == torch.float16 else tl.bfloat16,
        BIAS_LAST_SIZE=(rel_h_w.size(-1) // 2),
        B0_NUMEL=rel_h_w.size(-1),
        BLOCK_M=BLOCK_M,
        BLOCK_N=BLOCK_N,
        BLOCK_DMODEL=Lk,
        num_warps=num_warps,
        num_stages=num_stages)


def _load_best_configs():
    device_name = torch.cuda.get_device_name()
    if not device_name.startswith('NVIDIA A100'):
        print("Warning: Custom flash attention kernels were written specifically for A100.")
    import importlib
    saved_configs = importlib.resources.files("segment_anything_fast")
    saved_configs = saved_configs / "configs" / "flash_4_configs_a100.p"
    if not device_name.startswith('NVIDIA A100'):
        cwd = pathlib.Path.cwd()
        saved_configs = cwd / "flash_4_configs.p"
        print(f"We will try to read previously created kernel configurations from {saved_configs}.")
        print("You can disable this kernel by setting SEGMENT_ANYTHING_FAST_USE_FLASH_4=0")
        return None
    if saved_configs.is_file():
        import pickle
        with open(saved_configs, 'rb') as f:
            print(f"Loading best configs from file {saved_configs}")
            return pickle.load(f)


def _save_best_configs(best_configs):
    import importlib
    saved_configs = importlib.resources.files("segment_anything_fast")
    saved_configs = saved_configs / "configs" / "flash_4_configs_a100.p"
    device_name = torch.cuda.get_device_name()
    if not device_name.startswith('NVIDIA A100'):
        saved_configs = pathlib.Path.cwd() / "flash_4_configs.p"
        print("Warning: Custom flash attention kernels were written specifically for A100.")
        print(f"Storing configs for {device_name} locally under {saved_configs}")
    with open(saved_configs, 'wb') as f:
        import pickle
        print(f"Saving best configs to file {saved_configs}")
        pickle.dump(best_configs, f)


def _create_best_configs_key(q, k, v, rel_h_w, o):
    key = (q.size(),   k.size(),   v.size(),   rel_h_w.size(),   o.size(),
           q.stride(), k.stride(), v.stride(), rel_h_w.stride(), o.stride())
    return key


BEST_CONFIGS = None

lib = torch.library.Library("customflash", "FRAGMENT")
lib.define("custom_flash_aligned(Tensor q, Tensor k, Tensor v, Tensor rel_h_w, float sm_scale) -> Tensor")


# All that's needed for torch.compile support
@torch.library.impl(lib, "custom_flash_aligned", "Meta")
def _attention_rel_h_rel_w_kernel_aligned_meta(q, k, v, rel_h_w, sm_scale):
    return q.contiguous()


@torch.library.impl(lib, "custom_flash_aligned", "CUDA")
def _attention_rel_h_rel_w_kernel_aligned(q, k, v, rel_h_w, sm_scale):
    # This is likely not needed, but without it the kernel
    # is guaranteed to fail. If the inputs are already contiguous
    # these are cheap checks via is_contiguous and do nothing.
    q = q.contiguous()
    k = k.contiguous()
    v = v.contiguous()
    # shape constraints
    Lq, Lk, Lv = q.shape[-1], k.shape[-1], v.shape[-1]
    assert Lq == Lk and Lk == Lv
    assert Lk in {16, 32, 64, 128}
    o = torch.empty_like(q, memory_format=torch.contiguous_format)

    global BEST_CONFIGS
    if BEST_CONFIGS is None:
        BEST_CONFIGS = _load_best_configs()
    # Loading must have not been successful. Let's create a new dictionary.
    if BEST_CONFIGS is None:
        BEST_CONFIGS = {}
    key = _create_best_configs_key(q, k, v, rel_h_w, o)
    if key not in BEST_CONFIGS:
        print("key ", key, " not found. Running autotune. This might take a while.")
        import functools
        import itertools
        configs = []
        for (BLOCK_M, BLOCK_N, num_warps) in itertools.product([64, 128], [64, 128], [1, 2, 4, 8]):
            for num_stages in range(1, num_warps + 1):
                configs.append((BLOCK_M, BLOCK_N, num_warps, num_stages))
        print("all configs len: ", len(configs))
        best, best_config = _autotune(configs, functools.partial(_attention_rel_h_rel_w_kernel_aligned_device,
                                                                 q, k, v, rel_h_w, sm_scale, o))
        BEST_CONFIGS[key] = best_config
        print("Found best_config ", best_config,
              " with time ", best, " for key ", key)
        _save_best_configs(BEST_CONFIGS)
    best_config = BEST_CONFIGS[key]
    if best_config is None:
        return torch.tensor([])

    _attention_rel_h_rel_w_kernel_aligned_device(q,
                                                 k,
                                                 v,
                                                 rel_h_w,
                                                 sm_scale,
                                                 o,
                                                 best_config[0],
                                                 best_config[1],
                                                 best_config[2],
                                                 best_config[3])

    return o


USE_CUSTOM_KERNEL = bool(int(os.environ.get('SEGMENT_ANYTHING_FAST_USE_FLASH_4', 1)))


def _attention_rel_h_rel_w(q_, k_, v_, rel_h_, rel_w_):
    """
    Writing this as a composite allows torch.compile to fuse
    the needed padding into previous operations and memory
    allocations.
    """

    import math
    sm_scale = 1. / math.sqrt(q_.size(-1))
    # Check if second last dimension is multiple of 256
    q_size_2_padded = (((q_.size(-2) + 256 - 1) // 256) * 256) - q_.size(-2)

    def kernel_guards(q_, k_, v_):
        return (q_.dtype == torch.bfloat16 or q_.dtype == torch.float16) and q_.dtype == k_.dtype and k_.dtype == v_.dtype and USE_CUSTOM_KERNEL
    # vit_b and vit_l
    if q_size_2_padded == 0 and q_.size(-1) == 64 and kernel_guards(q_, k_, v_):
        rel_h_w = torch.cat([rel_h_.squeeze(-1), rel_w_.squeeze(-2)], dim=-1)
        o = torch.ops.customflash.custom_flash_aligned(
            q_, k_, v_, rel_h_w, sm_scale)
        if o.numel() > 0:
            return o
    # vit_h
    if q_size_2_padded == 0 and q_.size(-1) == 80 and kernel_guards(q_, k_, v_):
        # Only support multiples of 64, so need to pad
        q = torch.nn.functional.pad(q_, (0, 128 - 80, 0, 0), "constant", 0)
        k = torch.nn.functional.pad(k_, (0, 128 - 80, 0, 0), "constant", 0)
        v = torch.nn.functional.pad(v_, (0, 128 - 80, 0, 0), "constant", 0)
        rel_h_w = torch.cat([rel_h_.squeeze(-1), rel_w_.squeeze(-2)], dim=-1)
        o = torch.ops.customflash.custom_flash_aligned(
            q, k, v, rel_h_w, sm_scale)
        if o.numel() > 0:
            return o[:, :, :, :80]
    attn_bias = (rel_h_ + rel_w_).view(q_.size(0), q_.size(1),
                                       rel_h_.size(2), rel_h_.size(3) * rel_w_.size(4))
    return torch.nn.functional.scaled_dot_product_attention(q_, k_, v_, attn_mask=attn_bias)



# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtGui, QtCore, QtWidgets
from ISAT.ui.shortcut_dialog import Ui_Dialog

class ShortcutDialog(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, parent):
        super(ShortcutDialog, self).__init__(parent)
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)


# -*- coding: utf-8 -*-
# @Author  : LG

model_dict = {
    'mobile_sam.pt':
        {
            'urls': [
                'https://huggingface.co/yateng/isat_checkpoints/resolve/main/mobile_sam.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/mobile_sam.pt'
            ],
            'memory': '534M',
            'bf16_memory': '390M',
            'params': '40M'
        },
    'sam_hq_vit_h.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_h.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_h.pth'
            ],
            'memory': '6464M',
            'bf16_memory': '3378M',
            'params': '2.6G'
        },
    'sam_hq_vit_l.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_l.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_l.pth'
            ],
            'memory': '5016M',
            'bf16_memory': '2634M',
            'params': '1.3G'
        },
    'sam_hq_vit_b.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_b.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_b.pth'
            ],
            'memory': '3304M',
            'bf16_memory': '1762M',
            'params': '379M'
        },
    'sam_hq_vit_tiny.pth':
        {
            'urls': [
                'https://huggingface.co/lkeab/hq-sam/resolve/main/sam_hq_vit_tiny.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_hq_vit_tiny.pth'
            ],
            'memory': '598M',
            'bf16_memory': '392M',
            'params': '43M'
        },
    'sam_vit_h_4b8939.pth':
        {
            'urls': [
                'https://dl.fbaipublicfiles.com/segment_anything/sam_vit_h_4b8939.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_vit_h_4b8939.pth'
            ],
            'memory': '6462M',
            'bf16_memory': '3378M',
            'params': '2.6G'
        },
    'sam_vit_l_0b3195.pth':
        {
            'urls': [
                'https://dl.fbaipublicfiles.com/segment_anything/sam_vit_l_0b3195.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_vit_l_0b3195.pth'
            ],
            'memory': '5016M',
            'bf16_memory': '2634M',
            'params': '1.3G'
        },
    'sam_vit_b_01ec64.pth':
        {
            'urls': [
                'https://dl.fbaipublicfiles.com/segment_anything/sam_vit_b_01ec64.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam_vit_b_01ec64.pth'
            ],
            'memory': '3302M',
            'bf16_memory': '1760M',
            'params': '375M'
        },
    'edge_sam.pth':
        {
            'urls': [
                'https://huggingface.co/spaces/chongzhou/EdgeSAM/resolve/main/weights/edge_sam.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/edge_sam.pth'
            ],
            'memory': '360M',
            'bf16_memory': '304M',
            'params': '38.8M'
        },
    'edge_sam_3x.pth':
        {
            'urls': [
                'https://huggingface.co/spaces/chongzhou/EdgeSAM/resolve/main/weights/edge_sam_3x.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/edge_sam_3x.pth'
            ],
            'memory': '360M',
            'bf16_memory': '304M',
            'params': '38.8M'
        },
    'sam-med2d_b.pth':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam-med2d_b.pth',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam-med2d_b.pth'
            ],
            'memory': '1500M',
            'bf16_memory': '1050M',
            'params': '2.4G'
        },
    'sam2_hiera_large.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_large.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_large.pt'
            ],
            'memory': '4000M',
            'bf16_memory': '2800M',
            'params': '900M'
        },
    'sam2_hiera_base_plus.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_base_plus.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_base_plus.pt'
            ],
            'memory': '2800M',
            'bf16_memory': '2200M',
            'params': '324M'
        },
    'sam2_hiera_small.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_small.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_small.pt'
            ],
            'memory': '2500M',
            'bf16_memory': '1800M',
            'params': '185M'
        },
    'sam2_hiera_tiny.pt':
        {
            'urls': [
                'https://huggingface.co/yatengLG/ISAT_with_segment_anything_checkpoints/resolve/main/sam2_hiera_tiny.pt',
                'https://www.modelscope.cn/api/v1/models/yatengLG/ISAT_with_segment_anything_checkpoints/repo?Revision=master&FilePath=checkpoints/sam2_hiera_tiny.pt'
            ],
            'memory': '2200M',
            'bf16_memory': '1500M',
            'params': '156M'
        },
}



# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtWidgets, QtCore, QtGui
from ISAT.ui.info_dock import Ui_Form

class InfoDockWidget(QtWidgets.QWidget, Ui_Form):
    def __init__(self, mainwindow):
        super(InfoDockWidget, self).__init__()
        self.setupUi(self)
        self.mainwindow = mainwindow

        self.lineEdit_note.textChanged.connect(self.note_changed)

    def note_changed(self):
        if self.mainwindow.load_finished:
            self.mainwindow.set_saved_state(False)

    def update_widget(self):
        if self.mainwindow.current_label is not None:
            self.label_width.setText('{}'.format(self.mainwindow.current_label.width))
            self.label_height.setText('{}'.format(self.mainwindow.current_label.height))
            self.label_depth.setText('{}'.format(self.mainwindow.current_label.depth))
            self.lineEdit_note.setText(self.mainwindow.current_label.note)



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import logging

import torch
from hydra import compose
from hydra.utils import instantiate
from omegaconf import OmegaConf


def build_sam2(
    config_file,
    ckpt_path=None,
    device="cuda",
    mode="eval",
    hydra_overrides_extra=[],
    apply_postprocessing=True,
):

    if apply_postprocessing:
        hydra_overrides_extra = hydra_overrides_extra.copy()
        hydra_overrides_extra += [
            # dynamically fall back to multi-mask if the single mask is not stable
            "++model.sam_mask_decoder_extra_args.dynamic_multimask_via_stability=true",
            "++model.sam_mask_decoder_extra_args.dynamic_multimask_stability_delta=0.05",
            "++model.sam_mask_decoder_extra_args.dynamic_multimask_stability_thresh=0.98",
        ]
    # Read config and init model
    cfg = compose(config_name=config_file, overrides=hydra_overrides_extra)
    OmegaConf.resolve(cfg)
    model = instantiate(cfg.model, _recursive_=True)
    _load_checkpoint(model, ckpt_path)
    model = model.to(device)
    if mode == "eval":
        model.eval()
    return model


def build_sam2_hiera_tiny(checkpoint=None):
    return build_sam2(
        'sam2_hiera_t.yaml',
        ckpt_path=checkpoint,
        device="cuda",
        mode="eval",
        hydra_overrides_extra=[],
        apply_postprocessing=True,
    )


def build_sam2_hiera_small(checkpoint=None):
    return build_sam2(
        'sam2_hiera_s.yaml',
        ckpt_path=checkpoint,
        device="cuda",
        mode="eval",
        hydra_overrides_extra=[],
        apply_postprocessing=True,
    )


def build_sam2_hiera_base_plus(checkpoint=None):
    return build_sam2(
        'sam2_hiera_b+.yaml',
        ckpt_path=checkpoint,
        device="cuda",
        mode="eval",
        hydra_overrides_extra=[],
        apply_postprocessing=True,
    )


def build_sam2_hiera_large(checkpoint=None):
    return build_sam2(
        'sam2_hiera_l.yaml',
        ckpt_path=checkpoint,
        device="cuda",
        mode="eval",
        hydra_overrides_extra=[],
        apply_postprocessing=True,
    )


def build_sam2_video_predictor(
    config_file,
    ckpt_path=None,
    device="cuda",
    mode="eval",
    hydra_overrides_extra=[],
    apply_postprocessing=True,
):
    hydra_overrides = [
        "++model._target_=ISAT.segment_any.sam2.sam2_video_predictor.SAM2VideoPredictor",
    ]
    if apply_postprocessing:
        hydra_overrides_extra = hydra_overrides_extra.copy()
        hydra_overrides_extra += [
            # dynamically fall back to multi-mask if the single mask is not stable
            "++model.sam_mask_decoder_extra_args.dynamic_multimask_via_stability=true",
            "++model.sam_mask_decoder_extra_args.dynamic_multimask_stability_delta=0.05",
            "++model.sam_mask_decoder_extra_args.dynamic_multimask_stability_thresh=0.98",
            # the sigmoid mask logits on interacted frames with clicks in the memory encoder so that the encoded masks are exactly as what users see from clicking
            "++model.binarize_mask_from_pts_for_mem_enc=true",
            # fill small holes in the low-res masks up to `fill_hole_area` (before resizing them to the original video resolution)
            "++model.fill_hole_area=8",
        ]
    hydra_overrides.extend(hydra_overrides_extra)

    # Read config and init model
    cfg = compose(config_name=config_file, overrides=hydra_overrides)
    OmegaConf.resolve(cfg)
    model = instantiate(cfg.model, _recursive_=True)
    _load_checkpoint(model, ckpt_path)
    model = model.to(device)
    if mode == "eval":
        model.eval()
    return model


def _load_checkpoint(model, ckpt_path):
    if ckpt_path is not None:
        sd = torch.load(ckpt_path, map_location="cpu")["model"]
        missing_keys, unexpected_keys = model.load_state_dict(sd)
        if missing_keys:
            logging.error(missing_keys)
            raise RuntimeError()
        if unexpected_keys:
            logging.error(unexpected_keys)
            raise RuntimeError()
        logging.info("Loaded checkpoint sucessfully")


def build_sam2_video_predictor_hiera_tiny(checkpoint=None):
    return build_sam2_video_predictor(
        'sam2_hiera_t.yaml',
        ckpt_path=checkpoint,
    )


def build_sam2_video_predictor_hiera_small(checkpoint=None):
    return build_sam2_video_predictor(
        'sam2_hiera_s.yaml',
        ckpt_path=checkpoint,
    )


def build_sam2_video_predictor_hiera_base_plus(checkpoint=None):
    return build_sam2_video_predictor(
        'sam2_hiera_b+.yaml',
        ckpt_path=checkpoint,
    )


def build_sam2_video_predictor_hiera_large(checkpoint=None):
    return build_sam2_video_predictor(
        'sam2_hiera_l.yaml',
        ckpt_path=checkpoint,
    )


sam_model_registry = {
    "sam2_hiera_tiny": build_sam2_hiera_tiny,
    "sam2_hiera_small": build_sam2_hiera_small,
    "sam2_hiera_base_plus": build_sam2_hiera_base_plus,
    "sam2_hiera_large": build_sam2_hiera_large,

    "sam2_hiera_video_tiny": build_sam2_video_predictor_hiera_tiny,
    "sam2_hiera_video_small": build_sam2_video_predictor_hiera_small,
    "sam2_hiera_video_base_plus": build_sam2_video_predictor_hiera_base_plus,
    "sam2_hiera_video_large": build_sam2_video_predictor_hiera_large,
}


import yaml
from enum import Enum
import os

ISAT_ROOT = os.path.split(os.path.abspath(__file__))[0]
SOFTWARE_CONFIG_FILE = os.path.join(ISAT_ROOT, 'software.yaml')
CONFIG_FILE = os.path.join(ISAT_ROOT, 'isat.yaml')
CHECKPOINT_PATH = os.path.join(ISAT_ROOT, 'checkpoints')

os.makedirs(os.path.join(CHECKPOINT_PATH, 'tmp'), exist_ok=True)

if not os.path.exists(SOFTWARE_CONFIG_FILE):
    with open(SOFTWARE_CONFIG_FILE, 'w') as f:
        pass

if not os.path.exists(CONFIG_FILE):
    with open(CONFIG_FILE, 'w') as f:
        pass

def load_config(file):
    with open(file, 'rb')as f:
        cfg = yaml.load(f.read(), Loader=yaml.FullLoader)
    return cfg

def save_config(cfg, file):
    s = yaml.dump(cfg)
    with open(file, 'w') as f:
        f.write(s)
    return True

class STATUSMode(Enum):
    VIEW = 0
    CREATE = 1
    EDIT = 2
    REPAINT = 3

class DRAWMode(Enum):
    POLYGON = 0
    SEGMENTANYTHING = 1

class CLICKMode(Enum):
    POSITIVE = 0
    NEGATIVE = 1

class MAPMode(Enum):
    LABEL = 0
    SEMANTIC = 1
    INSTANCE = 2

class CONTOURMode(Enum):
    SAVE_MAX_ONLY = 0       # 只保留最多顶点的mask（一般为最大面积）
    SAVE_EXTERNAL = 1       # 只保留外轮廓
    SAVE_ALL = 2            # 保留所有轮廓


if __name__ == '__main__':
    cfg = load_config('/mnt/disk2/PycharmProjects/ISAT_with_segment_anything/ISAT/isat.yaml')



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'shortcut_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(726, 500)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Expanding)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(Dialog.sizePolicy().hasHeightForWidth())
        Dialog.setSizePolicy(sizePolicy)
        Dialog.setMinimumSize(QtCore.QSize(600, 300))
        Dialog.setMaximumSize(QtCore.QSize(1600, 500))
        font = QtGui.QFont()
        font.setKerning(True)
        Dialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setMinimumSize(QtCore.QSize(0, 60))
        self.widget.setMaximumSize(QtCore.QSize(16777215, 60))
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.label = QtWidgets.QLabel(self.widget)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(20)
        font.setBold(False)
        font.setItalic(False)
        font.setWeight(50)
        self.label.setFont(font)
        self.label.setTextFormat(QtCore.Qt.AutoText)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout.addWidget(self.widget)
        self.widget_2 = QtWidgets.QWidget(Dialog)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        self.widget_2.setFont(font)
        self.widget_2.setObjectName("widget_2")
        self.gridLayout = QtWidgets.QGridLayout(self.widget_2)
        self.gridLayout.setObjectName("gridLayout")
        self.label_51 = QtWidgets.QLabel(self.widget_2)
        self.label_51.setMaximumSize(QtCore.QSize(25, 25))
        self.label_51.setText("")
        self.label_51.setPixmap(QtGui.QPixmap(":/icon/icons/M_Favicon.ico"))
        self.label_51.setScaledContents(True)
        self.label_51.setObjectName("label_51")
        self.gridLayout.addWidget(self.label_51, 1, 0, 1, 1)
        self.label_28 = QtWidgets.QLabel(self.widget_2)
        self.label_28.setMinimumSize(QtCore.QSize(0, 25))
        self.label_28.setObjectName("label_28")
        self.gridLayout.addWidget(self.label_28, 4, 5, 1, 1)
        self.label_45 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_45.setFont(font)
        self.label_45.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_45.setAlignment(QtCore.Qt.AlignCenter)
        self.label_45.setObjectName("label_45")
        self.gridLayout.addWidget(self.label_45, 10, 7, 1, 1)
        self.label_35 = QtWidgets.QLabel(self.widget_2)
        self.label_35.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_35.setFont(font)
        self.label_35.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_35.setAlignment(QtCore.Qt.AlignCenter)
        self.label_35.setObjectName("label_35")
        self.gridLayout.addWidget(self.label_35, 3, 7, 1, 1)
        self.label_25 = QtWidgets.QLabel(self.widget_2)
        self.label_25.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_25.setFont(font)
        self.label_25.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_25.setAlignment(QtCore.Qt.AlignCenter)
        self.label_25.setObjectName("label_25")
        self.gridLayout.addWidget(self.label_25, 2, 7, 1, 1)
        self.label_5 = QtWidgets.QLabel(self.widget_2)
        self.label_5.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_5.setFont(font)
        self.label_5.setObjectName("label_5")
        self.gridLayout.addWidget(self.label_5, 1, 5, 1, 1)
        self.label_11 = QtWidgets.QLabel(self.widget_2)
        self.label_11.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_11.setFont(font)
        self.label_11.setObjectName("label_11")
        self.gridLayout.addWidget(self.label_11, 7, 5, 1, 1)
        self.label_9 = QtWidgets.QLabel(self.widget_2)
        self.label_9.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_9.setFont(font)
        self.label_9.setObjectName("label_9")
        self.gridLayout.addWidget(self.label_9, 6, 5, 1, 1)
        self.label_21 = QtWidgets.QLabel(self.widget_2)
        self.label_21.setMinimumSize(QtCore.QSize(0, 25))
        self.label_21.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_21.setFont(font)
        self.label_21.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_21.setAlignment(QtCore.Qt.AlignCenter)
        self.label_21.setObjectName("label_21")
        self.gridLayout.addWidget(self.label_21, 7, 7, 1, 1)
        self.label_30 = QtWidgets.QLabel(self.widget_2)
        self.label_30.setMinimumSize(QtCore.QSize(0, 25))
        self.label_30.setObjectName("label_30")
        self.gridLayout.addWidget(self.label_30, 8, 1, 1, 1)
        self.label_10 = QtWidgets.QLabel(self.widget_2)
        self.label_10.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_10.setFont(font)
        self.label_10.setObjectName("label_10")
        self.gridLayout.addWidget(self.label_10, 5, 5, 1, 1)
        self.label_33 = QtWidgets.QLabel(self.widget_2)
        self.label_33.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_33.setFont(font)
        self.label_33.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_33.setAlignment(QtCore.Qt.AlignCenter)
        self.label_33.setObjectName("label_33")
        self.gridLayout.addWidget(self.label_33, 8, 7, 1, 1)
        self.label_64 = QtWidgets.QLabel(self.widget_2)
        self.label_64.setMaximumSize(QtCore.QSize(25, 25))
        self.label_64.setText("")
        self.label_64.setPixmap(QtGui.QPixmap(":/icon/icons/校验_check-one.svg"))
        self.label_64.setScaledContents(True)
        self.label_64.setObjectName("label_64")
        self.gridLayout.addWidget(self.label_64, 2, 4, 1, 1)
        self.label_2 = QtWidgets.QLabel(self.widget_2)
        self.label_2.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_2.setFont(font)
        self.label_2.setObjectName("label_2")
        self.gridLayout.addWidget(self.label_2, 0, 1, 1, 1)
        self.label_16 = QtWidgets.QLabel(self.widget_2)
        self.label_16.setMinimumSize(QtCore.QSize(0, 25))
        self.label_16.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_16.setFont(font)
        self.label_16.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_16.setAlignment(QtCore.Qt.AlignCenter)
        self.label_16.setObjectName("label_16")
        self.gridLayout.addWidget(self.label_16, 7, 2, 1, 1)
        self.label_15 = QtWidgets.QLabel(self.widget_2)
        self.label_15.setMinimumSize(QtCore.QSize(0, 25))
        self.label_15.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_15.setFont(font)
        self.label_15.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_15.setAlignment(QtCore.Qt.AlignCenter)
        self.label_15.setObjectName("label_15")
        self.gridLayout.addWidget(self.label_15, 6, 2, 1, 1)
        self.label_29 = QtWidgets.QLabel(self.widget_2)
        self.label_29.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_29.setFont(font)
        self.label_29.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_29.setAlignment(QtCore.Qt.AlignCenter)
        self.label_29.setObjectName("label_29")
        self.gridLayout.addWidget(self.label_29, 4, 7, 1, 1)
        self.label_18 = QtWidgets.QLabel(self.widget_2)
        self.label_18.setMinimumSize(QtCore.QSize(0, 25))
        self.label_18.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_18.setFont(font)
        self.label_18.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_18.setAlignment(QtCore.Qt.AlignCenter)
        self.label_18.setObjectName("label_18")
        self.gridLayout.addWidget(self.label_18, 1, 7, 1, 1)
        self.label_27 = QtWidgets.QLabel(self.widget_2)
        self.label_27.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_27.setFont(font)
        self.label_27.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_27.setAlignment(QtCore.Qt.AlignCenter)
        self.label_27.setObjectName("label_27")
        self.gridLayout.addWidget(self.label_27, 4, 2, 1, 1)
        self.label_14 = QtWidgets.QLabel(self.widget_2)
        self.label_14.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_14.setFont(font)
        self.label_14.setObjectName("label_14")
        self.gridLayout.addWidget(self.label_14, 7, 1, 1, 1)
        self.label_12 = QtWidgets.QLabel(self.widget_2)
        self.label_12.setMinimumSize(QtCore.QSize(0, 25))
        self.label_12.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_12.setFont(font)
        self.label_12.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_12.setAlignment(QtCore.Qt.AlignCenter)
        self.label_12.setObjectName("label_12")
        self.gridLayout.addWidget(self.label_12, 1, 2, 1, 1)
        self.label_6 = QtWidgets.QLabel(self.widget_2)
        self.label_6.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_6.setFont(font)
        self.label_6.setObjectName("label_6")
        self.gridLayout.addWidget(self.label_6, 5, 1, 1, 1)
        self.label_24 = QtWidgets.QLabel(self.widget_2)
        self.label_24.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_24.setFont(font)
        self.label_24.setObjectName("label_24")
        self.gridLayout.addWidget(self.label_24, 2, 5, 1, 1)
        self.label_42 = QtWidgets.QLabel(self.widget_2)
        self.label_42.setObjectName("label_42")
        self.gridLayout.addWidget(self.label_42, 10, 1, 1, 1)
        self.label_38 = QtWidgets.QLabel(self.widget_2)
        self.label_38.setObjectName("label_38")
        self.gridLayout.addWidget(self.label_38, 9, 1, 1, 1)
        self.label_36 = QtWidgets.QLabel(self.widget_2)
        self.label_36.setObjectName("label_36")
        self.gridLayout.addWidget(self.label_36, 3, 1, 1, 1)
        self.label_43 = QtWidgets.QLabel(self.widget_2)
        self.label_43.setObjectName("label_43")
        self.gridLayout.addWidget(self.label_43, 10, 5, 1, 1)
        self.label_39 = QtWidgets.QLabel(self.widget_2)
        self.label_39.setObjectName("label_39")
        self.gridLayout.addWidget(self.label_39, 9, 5, 1, 1)
        self.label_34 = QtWidgets.QLabel(self.widget_2)
        self.label_34.setMinimumSize(QtCore.QSize(0, 25))
        self.label_34.setObjectName("label_34")
        self.gridLayout.addWidget(self.label_34, 3, 5, 1, 1)
        self.label_41 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_41.setFont(font)
        self.label_41.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_41.setAlignment(QtCore.Qt.AlignCenter)
        self.label_41.setObjectName("label_41")
        self.gridLayout.addWidget(self.label_41, 9, 7, 1, 1)
        self.label_44 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_44.setFont(font)
        self.label_44.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_44.setAlignment(QtCore.Qt.AlignCenter)
        self.label_44.setObjectName("label_44")
        self.gridLayout.addWidget(self.label_44, 10, 2, 1, 1)
        self.label_46 = QtWidgets.QLabel(self.widget_2)
        self.label_46.setObjectName("label_46")
        self.gridLayout.addWidget(self.label_46, 11, 1, 1, 1)
        self.label_47 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_47.setFont(font)
        self.label_47.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_47.setAlignment(QtCore.Qt.AlignCenter)
        self.label_47.setObjectName("label_47")
        self.gridLayout.addWidget(self.label_47, 11, 2, 1, 1)
        self.label_48 = QtWidgets.QLabel(self.widget_2)
        self.label_48.setObjectName("label_48")
        self.gridLayout.addWidget(self.label_48, 11, 5, 1, 1)
        self.label_49 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_49.setFont(font)
        self.label_49.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_49.setAlignment(QtCore.Qt.AlignCenter)
        self.label_49.setObjectName("label_49")
        self.gridLayout.addWidget(self.label_49, 11, 7, 1, 1)
        self.label_37 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_37.setFont(font)
        self.label_37.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_37.setAlignment(QtCore.Qt.AlignCenter)
        self.label_37.setObjectName("label_37")
        self.gridLayout.addWidget(self.label_37, 3, 2, 1, 1)
        self.label_32 = QtWidgets.QLabel(self.widget_2)
        self.label_32.setMinimumSize(QtCore.QSize(0, 25))
        self.label_32.setObjectName("label_32")
        self.gridLayout.addWidget(self.label_32, 8, 5, 1, 1)
        self.label_8 = QtWidgets.QLabel(self.widget_2)
        self.label_8.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_8.setFont(font)
        self.label_8.setObjectName("label_8")
        self.gridLayout.addWidget(self.label_8, 6, 1, 1, 1)
        self.label_17 = QtWidgets.QLabel(self.widget_2)
        self.label_17.setMinimumSize(QtCore.QSize(0, 25))
        self.label_17.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_17.setFont(font)
        self.label_17.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_17.setAlignment(QtCore.Qt.AlignCenter)
        self.label_17.setObjectName("label_17")
        self.gridLayout.addWidget(self.label_17, 0, 7, 1, 1)
        self.label_20 = QtWidgets.QLabel(self.widget_2)
        self.label_20.setMinimumSize(QtCore.QSize(0, 25))
        self.label_20.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_20.setFont(font)
        self.label_20.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_20.setAlignment(QtCore.Qt.AlignCenter)
        self.label_20.setObjectName("label_20")
        self.gridLayout.addWidget(self.label_20, 6, 7, 1, 1)
        self.label_26 = QtWidgets.QLabel(self.widget_2)
        self.label_26.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_26.setFont(font)
        self.label_26.setObjectName("label_26")
        self.gridLayout.addWidget(self.label_26, 4, 1, 1, 1)
        self.label_3 = QtWidgets.QLabel(self.widget_2)
        self.label_3.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_3.setFont(font)
        self.label_3.setObjectName("label_3")
        self.gridLayout.addWidget(self.label_3, 0, 5, 1, 1)
        self.label_22 = QtWidgets.QLabel(self.widget_2)
        self.label_22.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_22.setFont(font)
        self.label_22.setObjectName("label_22")
        self.gridLayout.addWidget(self.label_22, 2, 1, 1, 1)
        self.label_19 = QtWidgets.QLabel(self.widget_2)
        self.label_19.setMinimumSize(QtCore.QSize(0, 25))
        self.label_19.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_19.setFont(font)
        self.label_19.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_19.setAlignment(QtCore.Qt.AlignCenter)
        self.label_19.setObjectName("label_19")
        self.gridLayout.addWidget(self.label_19, 5, 7, 1, 1)
        self.label_7 = QtWidgets.QLabel(self.widget_2)
        self.label_7.setMinimumSize(QtCore.QSize(0, 25))
        self.label_7.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_7.setFont(font)
        self.label_7.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_7.setAlignment(QtCore.Qt.AlignCenter)
        self.label_7.setObjectName("label_7")
        self.gridLayout.addWidget(self.label_7, 0, 2, 1, 1)
        self.label_4 = QtWidgets.QLabel(self.widget_2)
        self.label_4.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setPointSize(12)
        self.label_4.setFont(font)
        self.label_4.setObjectName("label_4")
        self.gridLayout.addWidget(self.label_4, 1, 1, 1, 1)
        self.label_40 = QtWidgets.QLabel(self.widget_2)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_40.setFont(font)
        self.label_40.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_40.setAlignment(QtCore.Qt.AlignCenter)
        self.label_40.setObjectName("label_40")
        self.gridLayout.addWidget(self.label_40, 9, 2, 1, 1)
        self.label_13 = QtWidgets.QLabel(self.widget_2)
        self.label_13.setMinimumSize(QtCore.QSize(0, 25))
        self.label_13.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_13.setFont(font)
        self.label_13.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_13.setAlignment(QtCore.Qt.AlignCenter)
        self.label_13.setObjectName("label_13")
        self.gridLayout.addWidget(self.label_13, 5, 2, 1, 1)
        self.label_31 = QtWidgets.QLabel(self.widget_2)
        self.label_31.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_31.setFont(font)
        self.label_31.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_31.setAlignment(QtCore.Qt.AlignCenter)
        self.label_31.setObjectName("label_31")
        self.gridLayout.addWidget(self.label_31, 8, 2, 1, 1)
        self.label_23 = QtWidgets.QLabel(self.widget_2)
        self.label_23.setMinimumSize(QtCore.QSize(0, 25))
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        font.setBold(True)
        font.setWeight(75)
        self.label_23.setFont(font)
        self.label_23.setStyleSheet("background-color: #DFE1E5;\n"
"border-radius :5px;")
        self.label_23.setAlignment(QtCore.Qt.AlignCenter)
        self.label_23.setObjectName("label_23")
        self.gridLayout.addWidget(self.label_23, 2, 2, 1, 1)
        self.label_50 = QtWidgets.QLabel(self.widget_2)
        self.label_50.setMaximumSize(QtCore.QSize(25, 25))
        self.label_50.setText("")
        self.label_50.setPixmap(QtGui.QPixmap(":/icon/icons/上一步_back.svg"))
        self.label_50.setScaledContents(True)
        self.label_50.setObjectName("label_50")
        self.gridLayout.addWidget(self.label_50, 0, 0, 1, 1)
        self.label_55 = QtWidgets.QLabel(self.widget_2)
        self.label_55.setMaximumSize(QtCore.QSize(25, 25))
        self.label_55.setText("")
        self.label_55.setPixmap(QtGui.QPixmap(":/icon/icons/复制_copy.svg"))
        self.label_55.setScaledContents(True)
        self.label_55.setObjectName("label_55")
        self.gridLayout.addWidget(self.label_55, 3, 0, 1, 1)
        self.label_70 = QtWidgets.QLabel(self.widget_2)
        self.label_70.setMaximumSize(QtCore.QSize(25, 25))
        self.label_70.setText("")
        self.label_70.setPixmap(QtGui.QPixmap(":/icon/icons/下一步_next.svg"))
        self.label_70.setScaledContents(True)
        self.label_70.setObjectName("label_70")
        self.gridLayout.addWidget(self.label_70, 8, 4, 1, 1)
        self.label_63 = QtWidgets.QLabel(self.widget_2)
        self.label_63.setMaximumSize(QtCore.QSize(25, 25))
        self.label_63.setText("")
        self.label_63.setPixmap(QtGui.QPixmap(":/icon/icons/截屏_screenshot-two.svg"))
        self.label_63.setScaledContents(True)
        self.label_63.setObjectName("label_63")
        self.gridLayout.addWidget(self.label_63, 11, 0, 1, 1)
        self.label_62 = QtWidgets.QLabel(self.widget_2)
        self.label_62.setMaximumSize(QtCore.QSize(25, 25))
        self.label_62.setText("")
        self.label_62.setPixmap(QtGui.QPixmap(":/icon/icons/相交选择_intersect-selection.svg"))
        self.label_62.setScaledContents(True)
        self.label_62.setObjectName("label_62")
        self.gridLayout.addWidget(self.label_62, 10, 0, 1, 1)
        self.label_73 = QtWidgets.QLabel(self.widget_2)
        self.label_73.setMaximumSize(QtCore.QSize(25, 25))
        self.label_73.setText("")
        self.label_73.setPixmap(QtGui.QPixmap(":/icon/icons/截屏_screenshot-two.svg"))
        self.label_73.setScaledContents(True)
        self.label_73.setObjectName("label_73")
        self.gridLayout.addWidget(self.label_73, 11, 4, 1, 1)
        self.label_59 = QtWidgets.QLabel(self.widget_2)
        self.label_59.setMaximumSize(QtCore.QSize(25, 25))
        self.label_59.setText("")
        self.label_59.setPixmap(QtGui.QPixmap(":/icon/icons/照片_pic.svg"))
        self.label_59.setScaledContents(True)
        self.label_59.setObjectName("label_59")
        self.gridLayout.addWidget(self.label_59, 7, 0, 1, 1)
        self.label_72 = QtWidgets.QLabel(self.widget_2)
        self.label_72.setMaximumSize(QtCore.QSize(25, 25))
        self.label_72.setText("")
        self.label_72.setPixmap(QtGui.QPixmap(":/icon/icons/排除选择_exclude-selection.svg"))
        self.label_72.setScaledContents(True)
        self.label_72.setObjectName("label_72")
        self.gridLayout.addWidget(self.label_72, 10, 4, 1, 1)
        self.label_58 = QtWidgets.QLabel(self.widget_2)
        self.label_58.setMaximumSize(QtCore.QSize(25, 25))
        self.label_58.setText("")
        self.label_58.setPixmap(QtGui.QPixmap(":/icon/icons/去顶部_to-top.svg"))
        self.label_58.setScaledContents(True)
        self.label_58.setObjectName("label_58")
        self.gridLayout.addWidget(self.label_58, 6, 0, 1, 1)
        self.label_61 = QtWidgets.QLabel(self.widget_2)
        self.label_61.setMaximumSize(QtCore.QSize(25, 25))
        self.label_61.setText("")
        self.label_61.setPixmap(QtGui.QPixmap(":/icon/icons/合并选择_union-selection.svg"))
        self.label_61.setScaledContents(True)
        self.label_61.setObjectName("label_61")
        self.gridLayout.addWidget(self.label_61, 9, 0, 1, 1)
        self.label_54 = QtWidgets.QLabel(self.widget_2)
        self.label_54.setMaximumSize(QtCore.QSize(25, 25))
        self.label_54.setText("")
        self.label_54.setPixmap(QtGui.QPixmap(":/icon/icons/删除_delete-two.svg"))
        self.label_54.setScaledContents(True)
        self.label_54.setObjectName("label_54")
        self.gridLayout.addWidget(self.label_54, 2, 0, 1, 1)
        self.label_53 = QtWidgets.QLabel(self.widget_2)
        self.label_53.setMaximumSize(QtCore.QSize(25, 25))
        self.label_53.setText("")
        self.label_53.setPixmap(QtGui.QPixmap(":/icon/icons/锚点_anchor.svg"))
        self.label_53.setScaledContents(True)
        self.label_53.setObjectName("label_53")
        self.gridLayout.addWidget(self.label_53, 1, 4, 1, 1)
        self.label_52 = QtWidgets.QLabel(self.widget_2)
        self.label_52.setMaximumSize(QtCore.QSize(25, 25))
        self.label_52.setText("")
        self.label_52.setPixmap(QtGui.QPixmap(":/icon/icons/下一步_next.svg"))
        self.label_52.setScaledContents(True)
        self.label_52.setObjectName("label_52")
        self.gridLayout.addWidget(self.label_52, 0, 4, 1, 1)
        self.label_56 = QtWidgets.QLabel(self.widget_2)
        self.label_56.setMaximumSize(QtCore.QSize(25, 25))
        self.label_56.setText("")
        self.label_56.setPixmap(QtGui.QPixmap(":/icon/icons/关闭_close-one.svg"))
        self.label_56.setScaledContents(True)
        self.label_56.setObjectName("label_56")
        self.gridLayout.addWidget(self.label_56, 4, 0, 1, 1)
        self.label_57 = QtWidgets.QLabel(self.widget_2)
        self.label_57.setMaximumSize(QtCore.QSize(25, 25))
        self.label_57.setText("")
        self.label_57.setPixmap(QtGui.QPixmap(":/icon/icons/删除_delete.svg"))
        self.label_57.setScaledContents(True)
        self.label_57.setObjectName("label_57")
        self.gridLayout.addWidget(self.label_57, 5, 0, 1, 1)
        self.label_60 = QtWidgets.QLabel(self.widget_2)
        self.label_60.setMaximumSize(QtCore.QSize(25, 25))
        self.label_60.setText("")
        self.label_60.setPixmap(QtGui.QPixmap(":/icon/icons/上一步_back.svg"))
        self.label_60.setScaledContents(True)
        self.label_60.setObjectName("label_60")
        self.gridLayout.addWidget(self.label_60, 8, 0, 1, 1)
        self.label_71 = QtWidgets.QLabel(self.widget_2)
        self.label_71.setMaximumSize(QtCore.QSize(25, 25))
        self.label_71.setText("")
        self.label_71.setPixmap(QtGui.QPixmap(":/icon/icons/减去上一层_subtract-selection-one.svg"))
        self.label_71.setScaledContents(True)
        self.label_71.setObjectName("label_71")
        self.gridLayout.addWidget(self.label_71, 9, 4, 1, 1)
        self.label_68 = QtWidgets.QLabel(self.widget_2)
        self.label_68.setMaximumSize(QtCore.QSize(25, 25))
        self.label_68.setText("")
        self.label_68.setPixmap(QtGui.QPixmap(":/icon/icons/去底部_to-bottom.svg"))
        self.label_68.setScaledContents(True)
        self.label_68.setObjectName("label_68")
        self.gridLayout.addWidget(self.label_68, 6, 4, 1, 1)
        self.label_66 = QtWidgets.QLabel(self.widget_2)
        self.label_66.setMaximumSize(QtCore.QSize(25, 25))
        self.label_66.setText("")
        self.label_66.setPixmap(QtGui.QPixmap(":/icon/icons/眼睛_eyes.svg"))
        self.label_66.setScaledContents(True)
        self.label_66.setObjectName("label_66")
        self.gridLayout.addWidget(self.label_66, 4, 4, 1, 1)
        self.label_65 = QtWidgets.QLabel(self.widget_2)
        self.label_65.setMaximumSize(QtCore.QSize(25, 25))
        self.label_65.setText("")
        self.label_65.setPixmap(QtGui.QPixmap(":/icon/icons/编辑撰写_writing-fluently.svg"))
        self.label_65.setScaledContents(True)
        self.label_65.setObjectName("label_65")
        self.gridLayout.addWidget(self.label_65, 3, 4, 1, 1)
        self.label_67 = QtWidgets.QLabel(self.widget_2)
        self.label_67.setMaximumSize(QtCore.QSize(25, 25))
        self.label_67.setText("")
        self.label_67.setPixmap(QtGui.QPixmap(":/icon/icons/保存_save.svg"))
        self.label_67.setScaledContents(True)
        self.label_67.setObjectName("label_67")
        self.gridLayout.addWidget(self.label_67, 5, 4, 1, 1)
        self.label_69 = QtWidgets.QLabel(self.widget_2)
        self.label_69.setMaximumSize(QtCore.QSize(25, 25))
        self.label_69.setText("")
        self.label_69.setPixmap(QtGui.QPixmap(":/icon/icons/全宽_fullwidth.svg"))
        self.label_69.setScaledContents(True)
        self.label_69.setObjectName("label_69")
        self.gridLayout.addWidget(self.label_69, 7, 4, 1, 1)
        self.label_74 = QtWidgets.QLabel(self.widget_2)
        self.label_74.setMaximumSize(QtCore.QSize(25, 16777215))
        self.label_74.setText("")
        self.label_74.setObjectName("label_74")
        self.gridLayout.addWidget(self.label_74, 0, 3, 1, 1)
        self.verticalLayout.addWidget(self.widget_2)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Shortcut"))
        self.label.setText(_translate("Dialog", "Shortcut"))
        self.label_28.setText(_translate("Dialog", "Polygons Visible"))
        self.label_45.setText(_translate("Dialog", "Ctrl + E"))
        self.label_35.setText(_translate("Dialog", "R"))
        self.label_25.setText(_translate("Dialog", "E"))
        self.label_5.setText(_translate("Dialog", "Draw polygon"))
        self.label_11.setText(_translate("Dialog", "Zoom fit"))
        self.label_9.setText(_translate("Dialog", "To bottom"))
        self.label_21.setText(_translate("Dialog", "F"))
        self.label_30.setText(_translate("Dialog", "Prev group"))
        self.label_10.setText(_translate("Dialog", "Save annotation"))
        self.label_33.setText(_translate("Dialog", "Tab"))
        self.label_2.setText(_translate("Dialog", "Prev image"))
        self.label_16.setText(_translate("Dialog", "Space"))
        self.label_15.setText(_translate("Dialog", "T"))
        self.label_29.setText(_translate("Dialog", "V"))
        self.label_18.setText(_translate("Dialog", "C"))
        self.label_27.setText(_translate("Dialog", "Esc"))
        self.label_14.setText(_translate("Dialog", "Bit map"))
        self.label_12.setText(_translate("Dialog", "Q"))
        self.label_6.setText(_translate("Dialog", "Delete polygon"))
        self.label_24.setText(_translate("Dialog", "Annotate finish"))
        self.label_42.setText(_translate("Dialog", "Polygon intersect"))
        self.label_38.setText(_translate("Dialog", "Polygon union"))
        self.label_36.setText(_translate("Dialog", "Copy"))
        self.label_43.setText(_translate("Dialog", "Polygon exclude"))
        self.label_39.setText(_translate("Dialog", "Polygon subtract"))
        self.label_34.setText(_translate("Dialog", "Repaint"))
        self.label_41.setText(_translate("Dialog", "Ctrl + D"))
        self.label_44.setText(_translate("Dialog", "Ctrl + Q"))
        self.label_46.setText(_translate("Dialog", "Scene shot"))
        self.label_47.setText(_translate("Dialog", "P"))
        self.label_48.setText(_translate("Dialog", "Window shot"))
        self.label_49.setText(_translate("Dialog", "Ctrl + P"))
        self.label_37.setText(_translate("Dialog", "Ctrl + C"))
        self.label_32.setText(_translate("Dialog", "Next group"))
        self.label_8.setText(_translate("Dialog", "To top"))
        self.label_17.setText(_translate("Dialog", "D"))
        self.label_20.setText(_translate("Dialog", "B"))
        self.label_26.setText(_translate("Dialog", "Annotate cancel"))
        self.label_3.setText(_translate("Dialog", "Next image"))
        self.label_22.setText(_translate("Dialog", "Backspace"))
        self.label_19.setText(_translate("Dialog", "S"))
        self.label_7.setText(_translate("Dialog", "A"))
        self.label_4.setText(_translate("Dialog", "Segment anything"))
        self.label_40.setText(_translate("Dialog", "Ctrl + A"))
        self.label_13.setText(_translate("Dialog", "Del"))
        self.label_31.setText(_translate("Dialog", "`"))
        self.label_23.setText(_translate("Dialog", "Z"))




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'setting_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.7
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(607, 369)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        Dialog.setFont(font)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setObjectName("widget")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.widget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget_2 = QtWidgets.QWidget(self.widget)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_3.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label = QtWidgets.QLabel(self.widget_2)
        self.label.setMaximumSize(QtCore.QSize(80, 16777215))
        font = QtGui.QFont()
        font.setPointSize(20)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.label_config_file = QtWidgets.QLabel(self.widget_2)
        self.label_config_file.setText("")
        self.label_config_file.setAlignment(QtCore.Qt.AlignBottom|QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft)
        self.label_config_file.setObjectName("label_config_file")
        self.horizontalLayout_3.addWidget(self.label_config_file)
        self.verticalLayout.addWidget(self.widget_2)
        self.category_list_widget = QtWidgets.QListWidget(self.widget)
        self.category_list_widget.setObjectName("category_list_widget")
        self.verticalLayout.addWidget(self.category_list_widget)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.category_input = QtWidgets.QLineEdit(self.widget)
        self.category_input.setObjectName("category_input")
        self.horizontalLayout.addWidget(self.category_input)
        self.color_button = QtWidgets.QToolButton(self.widget)
        self.color_button.setStyleSheet("background-color: rgb(0, 255, 0);")
        self.color_button.setText("")
        self.color_button.setObjectName("color_button")
        self.horizontalLayout.addWidget(self.color_button)
        self.add_button = QtWidgets.QPushButton(self.widget)
        self.add_button.setMinimumSize(QtCore.QSize(150, 0))
        self.add_button.setSizeIncrement(QtCore.QSize(0, 0))
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/icons/书签_bookmark-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.add_button.setIcon(icon)
        self.add_button.setObjectName("add_button")
        self.horizontalLayout.addWidget(self.add_button)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.pushButton_import = QtWidgets.QPushButton(self.widget)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/icons/传入3_afferent-three.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_import.setIcon(icon1)
        self.pushButton_import.setObjectName("pushButton_import")
        self.horizontalLayout_2.addWidget(self.pushButton_import)
        self.pushButton_export = QtWidgets.QPushButton(self.widget)
        icon2 = QtGui.QIcon()
        icon2.addPixmap(QtGui.QPixmap(":/icons/icons/传出3_efferent-three.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.pushButton_export.setIcon(icon2)
        self.pushButton_export.setObjectName("pushButton_export")
        self.horizontalLayout_2.addWidget(self.pushButton_export)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.cancel_button = QtWidgets.QPushButton(self.widget)
        icon3 = QtGui.QIcon()
        icon3.addPixmap(QtGui.QPixmap(":/icons/icons/关闭_close-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.cancel_button.setIcon(icon3)
        self.cancel_button.setObjectName("cancel_button")
        self.horizontalLayout_2.addWidget(self.cancel_button)
        self.apply_button = QtWidgets.QPushButton(self.widget)
        icon4 = QtGui.QIcon()
        icon4.addPixmap(QtGui.QPixmap(":/icons/icons/校验_check-one.svg"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.apply_button.setIcon(icon4)
        self.apply_button.setObjectName("apply_button")
        self.horizontalLayout_2.addWidget(self.apply_button)
        self.verticalLayout.addLayout(self.horizontalLayout_2)
        self.verticalLayout_2.addWidget(self.widget)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Setting"))
        self.label.setText(_translate("Dialog", "Label"))
        self.add_button.setText(_translate("Dialog", "Add new label"))
        self.pushButton_import.setText(_translate("Dialog", "Import"))
        self.pushButton_export.setText(_translate("Dialog", "Export"))
        self.cancel_button.setText(_translate("Dialog", "Cancel"))
        self.apply_button.setText(_translate("Dialog", "Apply"))
# import icons_rc



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import math
from typing import Tuple

import torch
import torch.nn as nn
import torch.nn.functional as F

from ISAT.segment_any.sam2.modeling.sam2_utils import DropPath, get_clones, LayerNorm2d


class MaskDownSampler(nn.Module):
    """
    Progressively downsample a mask by total_stride, each time by stride.
    Note that LayerNorm is applied per *token*, like in ViT.

    With each downsample (by a factor stride**2), channel capacity increases by the same factor.
    In the end, we linearly project to embed_dim channels.
    """

    def __init__(
        self,
        embed_dim=256,
        kernel_size=4,
        stride=4,
        padding=0,
        total_stride=16,
        activation=nn.GELU,
    ):
        super().__init__()
        num_layers = int(math.log2(total_stride) // math.log2(stride))
        assert stride**num_layers == total_stride
        self.encoder = nn.Sequential()
        mask_in_chans, mask_out_chans = 1, 1
        for _ in range(num_layers):
            mask_out_chans = mask_in_chans * (stride**2)
            self.encoder.append(
                nn.Conv2d(
                    mask_in_chans,
                    mask_out_chans,
                    kernel_size=kernel_size,
                    stride=stride,
                    padding=padding,
                )
            )
            self.encoder.append(LayerNorm2d(mask_out_chans))
            self.encoder.append(activation())
            mask_in_chans = mask_out_chans

        self.encoder.append(nn.Conv2d(mask_out_chans, embed_dim, kernel_size=1))

    def forward(self, x):
        return self.encoder(x)


# Lightly adapted from ConvNext (https://github.com/facebookresearch/ConvNeXt)
class CXBlock(nn.Module):
    r"""ConvNeXt Block. There are two equivalent implementations:
    (1) DwConv -> LayerNorm (channels_first) -> 1x1 Conv -> GELU -> 1x1 Conv; all in (N, C, H, W)
    (2) DwConv -> Permute to (N, H, W, C); LayerNorm (channels_last) -> Linear -> GELU -> Linear; Permute back
    We use (2) as we find it slightly faster in PyTorch

    Args:
        dim (int): Number of input channels.
        drop_path (float): Stochastic depth rate. Default: 0.0
        layer_scale_init_value (float): Init value for Layer Scale. Default: 1e-6.
    """

    def __init__(
        self,
        dim,
        kernel_size=7,
        padding=3,
        drop_path=0.0,
        layer_scale_init_value=1e-6,
        use_dwconv=True,
    ):
        super().__init__()
        self.dwconv = nn.Conv2d(
            dim,
            dim,
            kernel_size=kernel_size,
            padding=padding,
            groups=dim if use_dwconv else 1,
        )  # depthwise conv
        self.norm = LayerNorm2d(dim, eps=1e-6)
        self.pwconv1 = nn.Linear(
            dim, 4 * dim
        )  # pointwise/1x1 convs, implemented with linear layers
        self.act = nn.GELU()
        self.pwconv2 = nn.Linear(4 * dim, dim)
        self.gamma = (
            nn.Parameter(layer_scale_init_value * torch.ones((dim)), requires_grad=True)
            if layer_scale_init_value > 0
            else None
        )
        self.drop_path = DropPath(drop_path) if drop_path > 0.0 else nn.Identity()

    def forward(self, x):
        input = x
        x = self.dwconv(x)
        x = self.norm(x)
        x = x.permute(0, 2, 3, 1)  # (N, C, H, W) -> (N, H, W, C)
        x = self.pwconv1(x)
        x = self.act(x)
        x = self.pwconv2(x)
        if self.gamma is not None:
            x = self.gamma * x
        x = x.permute(0, 3, 1, 2)  # (N, H, W, C) -> (N, C, H, W)

        x = input + self.drop_path(x)
        return x


class Fuser(nn.Module):
    def __init__(self, layer, num_layers, dim=None, input_projection=False):
        super().__init__()
        self.proj = nn.Identity()
        self.layers = get_clones(layer, num_layers)

        if input_projection:
            assert dim is not None
            self.proj = nn.Conv2d(dim, dim, kernel_size=1)

    def forward(self, x):
        # normally x: (N, C, H, W)
        x = self.proj(x)
        for layer in self.layers:
            x = layer(x)
        return x


class MemoryEncoder(nn.Module):
    def __init__(
        self,
        out_dim,
        mask_downsampler,
        fuser,
        position_encoding,
        in_dim=256,  # in_dim of pix_feats
    ):
        super().__init__()

        self.mask_downsampler = mask_downsampler

        self.pix_feat_proj = nn.Conv2d(in_dim, in_dim, kernel_size=1)
        self.fuser = fuser
        self.position_encoding = position_encoding
        self.out_proj = nn.Identity()
        if out_dim != in_dim:
            self.out_proj = nn.Conv2d(in_dim, out_dim, kernel_size=1)

    def forward(
        self,
        pix_feat: torch.Tensor,
        masks: torch.Tensor,
        skip_mask_sigmoid: bool = False,
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        ## Process masks
        # sigmoid, so that less domain shift from gt masks which are bool
        if not skip_mask_sigmoid:
            masks = F.sigmoid(masks)
        masks = self.mask_downsampler(masks)

        ## Fuse pix_feats and downsampled masks
        # in case the visual features are on CPU, cast them to CUDA
        pix_feat = pix_feat.to(masks.device)

        x = self.pix_feat_proj(pix_feat)
        x = x + masks
        x = self.fuser(x)
        x = self.out_proj(x)

        pos = self.position_encoding(x).to(x.dtype)

        return {"vision_features": x, "vision_pos_enc": [pos]}



# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import torch
import torch.nn as nn
import torch.nn.functional as F

from typing import Type


class MLPBlock(nn.Module):
    def __init__(
        self,
        embedding_dim: int,
        mlp_dim: int,
        act: Type[nn.Module] = nn.GELU,
    ) -> None:
        super().__init__()
        self.lin1 = nn.Linear(embedding_dim, mlp_dim)
        self.lin2 = nn.Linear(mlp_dim, embedding_dim)
        self.act = act()

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        return self.lin2(self.act(self.lin1(x)))


# From https://github.com/facebookresearch/detectron2/blob/main/detectron2/layers/batch_norm.py # noqa
# Itself from https://github.com/facebookresearch/ConvNeXt/blob/d1fa8f6fef0a165b27399986cc2bdacc92777e40/models/convnext.py#L119  # noqa
class LayerNorm2d(nn.Module):
    def __init__(self, num_channels: int, eps: float = 1e-6) -> None:
        super().__init__()
        self.weight = nn.Parameter(torch.ones(num_channels))
        self.bias = nn.Parameter(torch.zeros(num_channels))
        self.eps = eps

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        u = x.mean(1, keepdim=True)
        s = (x - u).pow(2).mean(1, keepdim=True)
        x = (x - u) / torch.sqrt(s + self.eps)
        x = self.weight[:, None, None] * x + self.bias[:, None, None]
        return x


def val2list(x: list or tuple or any, repeat_time=1) -> list:
    if isinstance(x, (list, tuple)):
        return list(x)
    return [x for _ in range(repeat_time)]


def val2tuple(x: list or tuple or any, min_len: int = 1, idx_repeat: int = -1) -> tuple:
    x = val2list(x)

    # repeat elements if necessary
    if len(x) > 0:
        x[idx_repeat:idx_repeat] = [x[idx_repeat] for _ in range(min_len - len(x))]

    return tuple(x)


def list_sum(x: list) -> any:
    return x[0] if len(x) == 1 else x[0] + list_sum(x[1:])


def resize(
        x: torch.Tensor,
        size: any or None = None,
        scale_factor=None,
        mode: str = "bicubic",
        align_corners: bool or None = False,
) -> torch.Tensor:
    if mode in ["bilinear", "bicubic"]:
        return F.interpolate(
            x,
            size=size,
            scale_factor=scale_factor,
            mode=mode,
            align_corners=align_corners,
        )
    elif mode in ["nearest", "area"]:
        return F.interpolate(x, size=size, scale_factor=scale_factor, mode=mode)
    else:
        raise NotImplementedError(f"resize(mode={mode}) not implemented.")


class UpSampleLayer(nn.Module):
    def __init__(
            self,
            mode="bicubic",
            size=None,
            factor=2,
            align_corners=False,
    ):
        super(UpSampleLayer, self).__init__()
        self.mode = mode
        self.size = val2list(size, 2) if size is not None else None
        self.factor = None if self.size is not None else factor
        self.align_corners = align_corners

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        return resize(x, self.size, self.factor, self.mode, self.align_corners)


class OpSequential(nn.Module):
    def __init__(self, op_list):
        super(OpSequential, self).__init__()
        valid_op_list = []
        for op in op_list:
            if op is not None:
                valid_op_list.append(op)
        self.op_list = nn.ModuleList(valid_op_list)

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        for op in self.op_list:
            x = op(x)
        return x


# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'file_dock.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Form(object):
    def setupUi(self, Form):
        Form.setObjectName("Form")
        Form.resize(264, 341)
        self.verticalLayout = QtWidgets.QVBoxLayout(Form)
        self.verticalLayout.setContentsMargins(2, 2, 2, 2)
        self.verticalLayout.setSpacing(0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.listWidget = QtWidgets.QListWidget(Form)
        self.listWidget.setObjectName("listWidget")
        self.verticalLayout.addWidget(self.listWidget)
        self.widget_state = QtWidgets.QWidget(Form)
        self.widget_state.setObjectName("widget_state")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_state)
        self.horizontalLayout_2.setContentsMargins(0, 6, 0, 6)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_prev_state = QtWidgets.QLabel(self.widget_state)
        self.label_prev_state.setMaximumSize(QtCore.QSize(16777215, 3))
        self.label_prev_state.setStyleSheet("background-color: rgb(153, 153, 153);")
        self.label_prev_state.setText("")
        self.label_prev_state.setObjectName("label_prev_state")
        self.horizontalLayout_2.addWidget(self.label_prev_state)
        self.label_current_state = QtWidgets.QLabel(self.widget_state)
        self.label_current_state.setMaximumSize(QtCore.QSize(16777215, 3))
        self.label_current_state.setStyleSheet("background-color: rgb(153, 153, 153);")
        self.label_current_state.setText("")
        self.label_current_state.setObjectName("label_current_state")
        self.horizontalLayout_2.addWidget(self.label_current_state)
        self.label_next_state = QtWidgets.QLabel(self.widget_state)
        self.label_next_state.setMinimumSize(QtCore.QSize(50, 0))
        self.label_next_state.setMaximumSize(QtCore.QSize(16777215, 3))
        self.label_next_state.setStyleSheet("background-color: rgb(153, 153, 153);")
        self.label_next_state.setText("")
        self.label_next_state.setObjectName("label_next_state")
        self.horizontalLayout_2.addWidget(self.label_next_state)
        self.verticalLayout.addWidget(self.widget_state)
        self.widget_num = QtWidgets.QWidget(Form)
        self.widget_num.setObjectName("widget_num")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget_num)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.lineEdit_jump = QtWidgets.QLineEdit(self.widget_num)
        self.lineEdit_jump.setObjectName("lineEdit_jump")
        self.horizontalLayout.addWidget(self.lineEdit_jump)
        self.label_current = QtWidgets.QLabel(self.widget_num)
        self.label_current.setText("")
        self.label_current.setObjectName("label_current")
        self.horizontalLayout.addWidget(self.label_current)
        self.label = QtWidgets.QLabel(self.widget_num)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.label_all = QtWidgets.QLabel(self.widget_num)
        self.label_all.setText("")
        self.label_all.setObjectName("label_all")
        self.horizontalLayout.addWidget(self.label_all)
        self.verticalLayout.addWidget(self.widget_num)

        self.retranslateUi(Form)
        QtCore.QMetaObject.connectSlotsByName(Form)

    def retranslateUi(self, Form):
        _translate = QtCore.QCoreApplication.translate
        Form.setWindowTitle(_translate("Form", "Form"))
        self.label_prev_state.setStatusTip(_translate("Form", "Prev image SAM state"))
        self.label_prev_state.setWhatsThis(_translate("Form", "Prev image SAM state."))
        self.label_current_state.setStatusTip(_translate("Form", "Current image SAM state"))
        self.label_current_state.setWhatsThis(_translate("Form", "Current image SAM state."))
        self.label_next_state.setStatusTip(_translate("Form", "Next image SAM state"))
        self.label_next_state.setWhatsThis(_translate("Form", "Next image SAM state."))
        self.lineEdit_jump.setToolTip(_translate("Form", "Jump to the image. Input name or index."))
        self.lineEdit_jump.setPlaceholderText(_translate("Form", "Jump to the image."))
        self.label.setText(_translate("Form", "/"))



# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtWidgets

class RightButtonMenu(QtWidgets.QMenu):
    def __init__(self, mainwindow):
        super(RightButtonMenu, self).__init__()
        self.mainwindow = mainwindow




# Copyright (c) Meta Platforms, Inc. and affiliates.
# All rights reserved.

# This source code is licensed under the license found in the
# LICENSE file in the root directory of this source tree.

import torch
from functools import partial
from .modeling import ImageEncoderViT, MaskDecoder, PromptEncoder, Sam, TwoWayTransformer
from torch.nn import functional as F

def build_sam_vit_h(args):
    return _build_sam(
        encoder_embed_dim=1280,
        encoder_depth=32,
        encoder_num_heads=16,
        encoder_global_attn_indexes=[7, 15, 23, 31],
        image_size=args.image_size,
        checkpoint=args.sam_checkpoint,
        encoder_adapter = args.encoder_adapter,
    )


build_sam = build_sam_vit_h


def build_sam_vit_l(args):
    return _build_sam(
        encoder_embed_dim=1024,
        encoder_depth=24,
        encoder_num_heads=16,
        encoder_global_attn_indexes=[5, 11, 17, 23],
        image_size=args.image_size,
        checkpoint=args.sam_checkpoint,
        encoder_adapter = args.encoder_adapter,
    )


def build_sam_vit_b(checkpoint):
    return _build_sam(
        encoder_embed_dim=768,
        encoder_depth=12,
        encoder_num_heads=12,
        encoder_global_attn_indexes=[2, 5, 8, 11],
        image_size=256,
        checkpoint=checkpoint,
        encoder_adapter = True,

    )


sam_model_registry = {
    "default": build_sam_vit_h,
    "vit_h": build_sam_vit_h,
    "vit_l": build_sam_vit_l,
    "vit_b": build_sam_vit_b,
}


def _build_sam(
    encoder_embed_dim,
    encoder_depth,
    encoder_num_heads,
    encoder_global_attn_indexes,
    image_size,
    checkpoint,
    encoder_adapter,
):
    prompt_embed_dim = 256
    image_size = image_size
    vit_patch_size = 16
    image_embedding_size = image_size // vit_patch_size
    sam = Sam(
        image_encoder=ImageEncoderViT(
            depth=encoder_depth,
            embed_dim=encoder_embed_dim,
            img_size=image_size,
            mlp_ratio=4,
            norm_layer=partial(torch.nn.LayerNorm, eps=1e-6),
            num_heads=encoder_num_heads,
            patch_size=vit_patch_size,
            qkv_bias=True,
            use_rel_pos = True,
            global_attn_indexes=encoder_global_attn_indexes,
            window_size=14,
            out_chans=prompt_embed_dim,
            adapter_train = encoder_adapter,
        ),
        prompt_encoder=PromptEncoder(
            embed_dim=prompt_embed_dim,
            image_embedding_size=(image_embedding_size, image_embedding_size),
            input_image_size=(image_size, image_size),
            mask_in_chans=16,
        ),
        mask_decoder=MaskDecoder(
            num_multimask_outputs=3,
            transformer=TwoWayTransformer(
                depth=2,
                embedding_dim=prompt_embed_dim,
                mlp_dim=2048,
                num_heads=8,
            ),
            transformer_dim=prompt_embed_dim,
            iou_head_depth=3,
            iou_head_hidden_dim=256,
        ),
        pixel_mean=[123.675, 116.28, 103.53],
        pixel_std=[58.395, 57.12, 57.375],
    )
    # sam.train()
    if checkpoint is not None:
        with open(checkpoint, "rb") as f:
            state_dict = torch.load(f, map_location="cpu")
        try:
            if 'model' in state_dict.keys():
                print(encoder_adapter)
                sam.load_state_dict(state_dict['model'], False)
            else:
                if image_size==1024 and encoder_adapter==True:
                    sam.load_state_dict(state_dict, False)
                else:
                    sam.load_state_dict(state_dict)
        except:
            print('*******interpolate')
            new_state_dict = load_from(sam, state_dict, image_size, vit_patch_size)   
            sam.load_state_dict(new_state_dict)
        print(f"*******load {checkpoint}")
        
    return sam


def load_from(sam, state_dicts, image_size, vit_patch_size):

    sam_dict = sam.state_dict()
    except_keys = ['mask_tokens', 'output_hypernetworks_mlps', 'iou_prediction_head']
    new_state_dict = {k: v for k, v in state_dicts.items() if
                      k in sam_dict.keys() and except_keys[0] not in k and except_keys[1] not in k and except_keys[2] not in k}
    pos_embed = new_state_dict['image_encoder.pos_embed']
    token_size = int(image_size // vit_patch_size)
    if pos_embed.shape[1] != token_size:
        # resize pos embedding, which may sacrifice the performance, but I have no better idea
        pos_embed = pos_embed.permute(0, 3, 1, 2)  # [b, c, h, w]
        pos_embed = F.interpolate(pos_embed, (token_size, token_size), mode='bilinear', align_corners=False)
        pos_embed = pos_embed.permute(0, 2, 3, 1)  # [b, h, w, c]
        new_state_dict['image_encoder.pos_embed'] = pos_embed
        rel_pos_keys = [k for k in sam_dict.keys() if 'rel_pos' in k]

        global_rel_pos_keys = [k for k in rel_pos_keys if 
                                                        '2' in k or 
                                                        '5' in k or 
                                                        '7' in k or 
                                                        '8' in k or 
                                                        '11' in k or 
                                                        '13' in k or
                                                        '15' in k or 
                                                        '23' in k or 
                                                        '31' in k] 
        # print(sam_dict)
        for k in global_rel_pos_keys:
            h_check, w_check = sam_dict[k].shape
            rel_pos_params = new_state_dict[k]
            h, w = rel_pos_params.shape
            rel_pos_params = rel_pos_params.unsqueeze(0).unsqueeze(0)
            if h != h_check or w != w_check:
                rel_pos_params = F.interpolate(rel_pos_params, (h_check, w_check), mode='bilinear', align_corners=False)

            new_state_dict[k] = rel_pos_params[0, 0, ...]

    sam_dict.update(new_state_dict)
    return sam_dict




# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'model_manager_dialog.ui'
#
# Created by: PyQt5 UI code generator 5.15.9
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_Dialog(object):
    def setupUi(self, Dialog):
        Dialog.setObjectName("Dialog")
        Dialog.resize(800, 360)
        font = QtGui.QFont()
        font.setFamily("Times New Roman")
        font.setPointSize(12)
        Dialog.setFont(font)
        self.verticalLayout = QtWidgets.QVBoxLayout(Dialog)
        self.verticalLayout.setContentsMargins(9, 9, 9, 9)
        self.verticalLayout.setObjectName("verticalLayout")
        self.widget_2 = QtWidgets.QWidget(Dialog)
        self.widget_2.setObjectName("widget_2")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.widget_2)
        self.horizontalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_2.addItem(spacerItem)
        self.checkBox_use_bfloat16 = QtWidgets.QCheckBox(self.widget_2)
        self.checkBox_use_bfloat16.setObjectName("checkBox_use_bfloat16")
        self.horizontalLayout_2.addWidget(self.checkBox_use_bfloat16)
        self.verticalLayout.addWidget(self.widget_2)
        self.tableWidget = QtWidgets.QTableWidget(Dialog)
        self.tableWidget.setFrameShape(QtWidgets.QFrame.Box)
        self.tableWidget.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.tableWidget.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setSelectionMode(QtWidgets.QAbstractItemView.SingleSelection)
        self.tableWidget.setSelectionBehavior(QtWidgets.QAbstractItemView.SelectRows)
        self.tableWidget.setShowGrid(True)
        self.tableWidget.setGridStyle(QtCore.Qt.SolidLine)
        self.tableWidget.setWordWrap(True)
        self.tableWidget.setCornerButtonEnabled(True)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(4)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(1, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(2, item)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(3, item)
        self.tableWidget.horizontalHeader().setCascadingSectionResizes(True)
        self.tableWidget.horizontalHeader().setHighlightSections(True)
        self.tableWidget.horizontalHeader().setSortIndicatorShown(False)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)
        self.tableWidget.verticalHeader().setVisible(False)
        self.tableWidget.verticalHeader().setStretchLastSection(False)
        self.verticalLayout.addWidget(self.tableWidget)
        self.widget = QtWidgets.QWidget(Dialog)
        self.widget.setObjectName("widget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.widget)
        self.horizontalLayout.setContentsMargins(0, 0, 0, 0)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.pushButton_clear_tmp = QtWidgets.QPushButton(self.widget)
        self.pushButton_clear_tmp.setObjectName("pushButton_clear_tmp")
        self.horizontalLayout.addWidget(self.pushButton_clear_tmp)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.label = QtWidgets.QLabel(self.widget)
        self.label.setObjectName("label")
        self.horizontalLayout.addWidget(self.label)
        self.verticalLayout.addWidget(self.widget)

        self.retranslateUi(Dialog)
        QtCore.QMetaObject.connectSlotsByName(Dialog)

    def retranslateUi(self, Dialog):
        _translate = QtCore.QCoreApplication.translate
        Dialog.setWindowTitle(_translate("Dialog", "Model Manage"))
        self.checkBox_use_bfloat16.setToolTip(_translate("Dialog", "use bfloat16 to reduce GPU memory required"))
        self.checkBox_use_bfloat16.setText(_translate("Dialog", "use bfloat16"))
        self.tableWidget.setSortingEnabled(False)
        item = self.tableWidget.horizontalHeaderItem(0)
        item.setText(_translate("Dialog", "model"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_translate("Dialog", "memory(GPU)"))
        item = self.tableWidget.horizontalHeaderItem(2)
        item.setText(_translate("Dialog", "params"))
        item = self.tableWidget.horizontalHeaderItem(3)
        item.setText(_translate("Dialog", "operate"))
        self.pushButton_clear_tmp.setText(_translate("Dialog", "clear tmp"))
        self.label.setText(_translate("Dialog", "Download model, and select it in SAM menu to use."))



import torch
from torch.sparse import to_sparse_semi_structured, SparseSemiStructuredTensor
SparseSemiStructuredTensor._FORCE_CUTLASS = True

# Sparsity helper functions
def apply_fake_sparsity(model):
    """
    This function simulates 2:4 sparsity on all linear layers in a model.
    It uses the torch.ao.pruning flow.
    """
    # torch.ao.pruning flow
    from torch.ao.pruning import WeightNormSparsifier
    sparse_config = []
    for name, mod in model.named_modules():
        if isinstance(mod, torch.nn.Linear):
            sparse_config.append({"tensor_fqn": f"{name}.weight"})

    sparsifier = WeightNormSparsifier(sparsity_level=1.0,
                                      sparse_block_shape=(1,4),
                                      zeros_per_block=2)
    sparsifier.prepare(model, sparse_config)
    sparsifier.step()

    sparsifier.step()
    sparsifier.squash_mask()


def apply_sparse(model):
    apply_fake_sparsity(model)
    for name, mod in model.named_modules():
        if isinstance(mod, torch.nn.Linear):
            mod.weight = torch.nn.Parameter(to_sparse_semi_structured(mod.weight))



# -*- coding: utf-8 -*-
# @Author  : LG


import torch
import numpy as np
import timm
import platform
from PIL import Image
from collections import OrderedDict
import os
from skimage.draw.draw import polygon
from ISAT.segment_any.sam2.utils.misc import AsyncVideoFrameLoader


osplatform = platform.system()

class SegAny:
    def __init__(self, checkpoint:str, use_bfloat16:bool=True):
        print('--' * 20)
        print('* Init SAM... *')
        self.checkpoint = checkpoint
        self.model_dtype = torch.bfloat16 if use_bfloat16 else torch.float32
        self.model_source = None
        if 'mobile_sam' in checkpoint:
            # mobile sam
            from ISAT.segment_any.mobile_sam import sam_model_registry, SamPredictor
            self.model_type = "vit_t"
            self.model_source = 'mobile_sam'
        elif 'edge_sam' in checkpoint:
            # edge_sam
            from ISAT.segment_any.edge_sam import sam_model_registry, SamPredictor
            self.model_type = "edge_sam"
            self.model_source = 'edge_sam'
        elif 'sam_hq_vit' in checkpoint:
            # sam hq
            from ISAT.segment_any.segment_anything_hq import sam_model_registry, SamPredictor
            if 'vit_b' in checkpoint:
                self.model_type = "vit_b"
            elif 'vit_l' in checkpoint:
                self.model_type = "vit_l"
            elif 'vit_h' in checkpoint:
                self.model_type = "vit_h"
            elif 'vit_tiny' in checkpoint:
                self.model_type = "vit_tiny"
            else:
                raise ValueError('The checkpoint named {} is not supported.'.format(checkpoint))
            self.model_source = 'sam_hq'

        elif 'sam_vit' in checkpoint:
            # sam
            if torch.__version__ > '2.1.1' and osplatform == 'Linux':
                # 暂时只测试了2.1.1环境下的运行;2.0不确定；1.x不可以
                # 暂时不使用sam-fast
                # from ISAT.segment_anything_fast import sam_model_registry as sam_model_registry
                # from ISAT.segment_anything_fast import SamPredictor
                # print('segment_anything_fast')
                from ISAT.segment_any.segment_anything import sam_model_registry, SamPredictor
                print('segment_anything')
            else:
                # windows下，现只支持 2.2.0+dev，且需要其他依赖；等后续正式版本推出后，再进行支持
                # （如果想提前在windows下试用，可参考https://github.com/pytorch-labs/segment-anything-fast项目进行环境配置）
                from ISAT.segment_any.segment_anything import sam_model_registry, SamPredictor
                print('segment_anything')
            if 'vit_b' in checkpoint:
                self.model_type = "vit_b"
            elif 'vit_l' in checkpoint:
                self.model_type = "vit_l"
            elif 'vit_h' in checkpoint:
                self.model_type = "vit_h"
            else:
                raise ValueError('The checkpoint named {} is not supported.'.format(checkpoint))
            self.model_source = 'sam'
        elif 'sam2' in checkpoint:
            from ISAT.segment_any.sam2.build_sam import sam_model_registry
            from ISAT.segment_any.sam2.sam2_image_predictor import SAM2ImagePredictor as SamPredictor
            # sam2
            if 'hiera_tiny' in checkpoint:
                self.model_type = "sam2_hiera_tiny"
            elif 'hiera_small' in checkpoint:
                self.model_type = "sam2_hiera_small"
            elif 'hiera_base_plus' in checkpoint:
                self.model_type = 'sam2_hiera_base_plus'
            elif 'hiera_large' in checkpoint:
                self.model_type = 'sam2_hiera_large'
            else:
                raise ValueError('The checkpoint named {} is not supported.'.format(checkpoint))
            self.model_source = 'sam2'
            # sam2 在float32下运行时存在报错，暂时只在bfloat16下运行
            # self.model_dtype = torch.bfloat16

        elif 'med2d' in checkpoint:
            from ISAT.segment_any.segment_anything_med2d import sam_model_registry
            from ISAT.segment_any.segment_anything_med2d.predictor_for_isat import Predictor as SamPredictor
            self.model_type = "vit_b"
            self.model_source = 'sam_med2d'

        torch.cuda.empty_cache()

        self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        print('  - device  : {}'.format(self.device))
        print('  - dtype   : {}'.format(self.model_dtype))
        print('  - loading : {}'.format(checkpoint))
        sam = sam_model_registry[self.model_type](checkpoint=checkpoint)

        sam = sam.eval().to(self.model_dtype)

        sam.to(device=self.device)
        self.predictor_with_point_prompt = SamPredictor(sam)
        print('* Init SAM finished *')
        print('--'*20)
        self.image = None

    def set_image(self, image):
        with torch.inference_mode(), torch.autocast(self.device, dtype=self.model_dtype):
            self.image = image
            self.predictor_with_point_prompt.set_image(image)

    def reset_image(self):
        self.predictor_with_point_prompt.reset_image()
        self.image = None
        torch.cuda.empty_cache()

    def predict_with_point_prompt(self, input_point, input_label):
        with torch.inference_mode(), torch.autocast(self.device, dtype=self.model_dtype):

            if 'sam2' not in self.model_type:
                input_point = np.array(input_point)
                input_label = np.array(input_label)
            else:
                input_point = input_point
                input_label = input_label

            masks, scores, logits = self.predictor_with_point_prompt.predict(
                point_coords=input_point,
                point_labels=input_label,
                multimask_output=True,
            )
            if self.model_source == 'sam_med2d':
                return masks

            mask_input = logits[np.argmax(scores), :, :]  # Choose the model's best mask
            masks, _, _ = self.predictor_with_point_prompt.predict(
                point_coords=input_point,
                point_labels=input_label,
                mask_input=mask_input[None, :, :],
                multimask_output=False,
            )
            torch.cuda.empty_cache()
            return masks

    def predict_with_box_prompt(self, box):
        with torch.inference_mode(), torch.autocast(self.device, dtype=self.model_dtype):
            masks, scores, logits = self.predictor_with_point_prompt.predict(
                box=box,
                multimask_output=False,
            )
            torch.cuda.empty_cache()
            return masks


class SegAnyVideo:
    def __init__(self, checkpoint: str, use_bfloat16: bool = True):
        print('--'*20)
        print('* Init SAM for video... *')

        self.checkpoint = checkpoint
        self.model_dtype = torch.bfloat16 if use_bfloat16 else torch.float32
        self.model_source = None

        self.inference_state = {}

        if 'sam2' in checkpoint:
            from ISAT.segment_any.sam2.build_sam import sam_model_registry
            # sam2
            if 'hiera_tiny' in checkpoint:
                self.model_type = "sam2_hiera_video_tiny"
            elif 'hiera_small' in checkpoint:
                self.model_type = "sam2_hiera_video_small"
            elif 'hiera_base_plus' in checkpoint:
                self.model_type = 'sam2_hiera_video_base_plus'
            elif 'hiera_large' in checkpoint:
                self.model_type = 'sam2_hiera_video_large'
            else:
                raise ValueError('The checkpoint named {} is not supported.'.format(checkpoint))
            self.model_source = 'sam2'

        torch.cuda.empty_cache()

        self.device = 'cuda' if torch.cuda.is_available() else 'cpu'
        print('  - device  : {}'.format(self.device))
        print('  - dtype   : {}'.format(self.model_dtype))
        print('  - loading : {}'.format(checkpoint))
        self.predictor = sam_model_registry[self.model_type](checkpoint=checkpoint)
        self.predictor = self.predictor.eval().to(self.model_dtype)
        self.predictor.to(device=self.device)
        print('* Init SAM for video finished *')
        print('--'*20)

    def init_state(
            self,
            image_root,
            image_name_list,
            offload_video_to_cpu=True,
            offload_state_to_cpu=True,
            async_loading_frames=False,
    ):
        with torch.inference_mode(), torch.autocast(self.device, dtype=self.model_dtype):

            img_mean = (0.485, 0.456, 0.406)
            img_std = (0.229, 0.224, 0.225)

            num_frames = len(image_name_list)
            img_paths = [os.path.join(image_root, frame_name) for frame_name in image_name_list]
            img_mean = torch.tensor(img_mean, dtype=torch.float32)[:, None, None]
            img_std = torch.tensor(img_std, dtype=torch.float32)[:, None, None]

            image_size = self.predictor.image_size

            if async_loading_frames:
                lazy_images = AsyncVideoFrameLoader(
                    img_paths, image_size, offload_video_to_cpu, img_mean, img_std
                )
                # return lazy_images, lazy_images.video_height, lazy_images.video_width
                images, video_height, video_width = lazy_images, lazy_images.video_height, lazy_images.video_width
            else:
                images = torch.zeros(num_frames, 3, image_size, image_size, dtype=torch.float32)
                for n, img_path in enumerate(img_paths):
                    images[n], video_height, video_width = self._load_img_as_tensor(img_path, image_size)
                if not offload_video_to_cpu:
                    images = images.cuda()
                    img_mean = img_mean.cuda()
                    img_std = img_std.cuda()

                images -= img_mean
                images /= img_std
                # images, video_height, video_width = load_video_frames

            inference_state = {}
            inference_state["images"] = images
            inference_state["num_frames"] = len(images)
            # whether to offload the video frames to CPU memory
            # turning on this option saves the GPU memory with only a very small overhead
            inference_state["offload_video_to_cpu"] = offload_video_to_cpu
            # whether to offload the inference state to CPU memory
            # turning on this option saves the GPU memory at the cost of a lower tracking fps
            # (e.g. in a test case of 768x768 model, fps dropped from 27 to 24 when tracking one object
            # and from 24 to 21 when tracking two objects)
            inference_state["offload_state_to_cpu"] = offload_state_to_cpu
            # the original video height and width, used for resizing final output scores
            inference_state["video_height"] = video_height
            inference_state["video_width"] = video_width
            inference_state["device"] = torch.device("cuda")
            if offload_state_to_cpu:
                inference_state["storage_device"] = torch.device("cpu")
            else:
                inference_state["storage_device"] = torch.device("cuda")
            # inputs on each frame
            inference_state["point_inputs_per_obj"] = {}
            inference_state["mask_inputs_per_obj"] = {}
            # visual features on a small number of recently visited frames for quick interactions
            inference_state["cached_features"] = {}
            # values that don't change across frames (so we only need to hold one copy of them)
            inference_state["constants"] = {}
            # mapping between client-side object id and model-side object index
            inference_state["obj_id_to_idx"] = OrderedDict()
            inference_state["obj_idx_to_id"] = OrderedDict()
            inference_state["obj_ids"] = []
            # A storage to hold the model's tracking results and states on each frame
            inference_state["output_dict"] = {
                "cond_frame_outputs": {},  # dict containing {frame_idx: <out>}
                "non_cond_frame_outputs": {},  # dict containing {frame_idx: <out>}
            }
            # Slice (view) of each object tracking results, sharing the same memory with "output_dict"
            inference_state["output_dict_per_obj"] = {}
            # A temporary storage to hold new outputs when user interact with a frame
            # to add clicks or mask (it's merged into "output_dict" before propagation starts)
            inference_state["temp_output_dict_per_obj"] = {}
            # Frames that already holds consolidated outputs from click or mask inputs
            # (we directly use their consolidated outputs during tracking)
            inference_state["consolidated_frame_inds"] = {
                "cond_frame_outputs": set(),  # set containing frame indices
                "non_cond_frame_outputs": set(),  # set containing frame indices
            }
            # metadata for each tracking frame (e.g. which direction it's tracked)
            inference_state["tracking_has_started"] = False
            inference_state["frames_already_tracked"] = {}
            # Warm up the visual backbone and cache the image feature on frame 0
            self.predictor._get_image_feature(inference_state, frame_idx=0, batch_size=1)

            self.inference_state = inference_state
            self.reset_state()

            print('init state finished.')

    def reset_state(self):
        """Remove all input points or mask in all frames throughout the video."""
        self._reset_tracking_results(self.inference_state)
        # Remove all object ids
        self.inference_state["obj_id_to_idx"].clear()
        self.inference_state["obj_idx_to_id"].clear()
        self.inference_state["obj_ids"].clear()
        self.inference_state["point_inputs_per_obj"].clear()
        self.inference_state["mask_inputs_per_obj"].clear()
        self.inference_state["output_dict_per_obj"].clear()
        self.inference_state["temp_output_dict_per_obj"].clear()

    def _reset_tracking_results(self, inference_state):
        """Reset all tracking inputs and results across the videos."""
        for v in inference_state["point_inputs_per_obj"].values():
            v.clear()
        for v in inference_state["mask_inputs_per_obj"].values():
            v.clear()
        for v in inference_state["output_dict_per_obj"].values():
            v["cond_frame_outputs"].clear()
            v["non_cond_frame_outputs"].clear()
        for v in inference_state["temp_output_dict_per_obj"].values():
            v["cond_frame_outputs"].clear()
            v["non_cond_frame_outputs"].clear()
        inference_state["output_dict"]["cond_frame_outputs"].clear()
        inference_state["output_dict"]["non_cond_frame_outputs"].clear()
        inference_state["consolidated_frame_inds"]["cond_frame_outputs"].clear()
        inference_state["consolidated_frame_inds"]["non_cond_frame_outputs"].clear()
        inference_state["tracking_has_started"] = False
        inference_state["frames_already_tracked"].clear()

    @staticmethod
    def _load_img_as_tensor(img_path, image_size):
        img_pil = Image.open(img_path)
        img_np = np.array(img_pil.convert("RGB").resize((image_size, image_size)))
        if img_np.dtype == np.uint8:  # np.uint8 is expected for JPEG images
            img_np = img_np / 255.0
        else:
            raise RuntimeError(f"Unknown image dtype: {img_np.dtype} on {img_path}")
        img = torch.from_numpy(img_np).permute(2, 0, 1)
        video_width, video_height = img_pil.size  # the original video size
        return img, video_height, video_width

    def add_new_mask(self, frame_idx, ann_obj_id, mask):
        with torch.inference_mode(), torch.autocast(self.device, dtype=self.model_dtype):
            self.predictor.add_new_mask(
                inference_state=self.inference_state,
                frame_idx=frame_idx,
                obj_id=ann_obj_id,
                mask=mask
            )



import torch.nn as nn
from .common import LayerNorm2d, UpSampleLayer, OpSequential

__all__ = ['rep_vit_m1', 'rep_vit_m2', 'rep_vit_m3', 'RepViT']

m1_cfgs = [
    # k, t, c, SE, HS, s
    [3, 2, 48, 1, 0, 1],
    [3, 2, 48, 0, 0, 1],
    [3, 2, 48, 0, 0, 1],
    [3, 2, 96, 0, 0, 2],
    [3, 2, 96, 1, 0, 1],
    [3, 2, 96, 0, 0, 1],
    [3, 2, 96, 0, 0, 1],
    [3, 2, 192, 0, 1, 2],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 1, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 192, 0, 1, 1],
    [3, 2, 384, 0, 1, 2],
    [3, 2, 384, 1, 1, 1],
    [3, 2, 384, 0, 1, 1]
]

m2_cfgs = [
    # k, t, c, SE, HS, s
    [3, 2, 64, 1, 0, 1],
    [3, 2, 64, 0, 0, 1],
    [3, 2, 64, 0, 0, 1],
    [3, 2, 128, 0, 0, 2],
    [3, 2, 128, 1, 0, 1],
    [3, 2, 128, 0, 0, 1],
    [3, 2, 128, 0, 0, 1],
    [3, 2, 256, 0, 1, 2],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 512, 0, 1, 2],
    [3, 2, 512, 1, 1, 1],
    [3, 2, 512, 0, 1, 1]
]

m3_cfgs = [
    # k, t, c, SE, HS, s
    [3, 2, 64, 1, 0, 1],
    [3, 2, 64, 0, 0, 1],
    [3, 2, 64, 1, 0, 1],
    [3, 2, 64, 0, 0, 1],
    [3, 2, 64, 0, 0, 1],
    [3, 2, 128, 0, 0, 2],
    [3, 2, 128, 1, 0, 1],
    [3, 2, 128, 0, 0, 1],
    [3, 2, 128, 1, 0, 1],
    [3, 2, 128, 0, 0, 1],
    [3, 2, 128, 0, 0, 1],
    [3, 2, 256, 0, 1, 2],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 1, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 256, 0, 1, 1],
    [3, 2, 512, 0, 1, 2],
    [3, 2, 512, 1, 1, 1],
    [3, 2, 512, 0, 1, 1]
]


def _make_divisible(v, divisor, min_value=None):
    """
    This function is taken from the original tf repo.
    It ensures that all layers have a channel number that is divisible by 8
    It can be seen here:
    https://github.com/tensorflow/models/blob/master/research/slim/nets/mobilenet/mobilenet.py
    :param v:
    :param divisor:
    :param min_value:
    :return:
    """
    if min_value is None:
        min_value = divisor
    new_v = max(min_value, int(v + divisor / 2) // divisor * divisor)
    # Make sure that round down does not go down by more than 10%.
    if new_v < 0.9 * v:
        new_v += divisor
    return new_v


from timm.models.layers import SqueezeExcite

import torch


class Conv2d_BN(torch.nn.Sequential):
    def __init__(self, a, b, ks=1, stride=1, pad=0, dilation=1,
                 groups=1, bn_weight_init=1, resolution=-10000):
        super().__init__()
        self.add_module('c', torch.nn.Conv2d(
            a, b, ks, stride, pad, dilation, groups, bias=False))
        self.add_module('bn', torch.nn.BatchNorm2d(b))
        torch.nn.init.constant_(self.bn.weight, bn_weight_init)
        torch.nn.init.constant_(self.bn.bias, 0)

    @torch.no_grad()
    def fuse(self):
        c, bn = self._modules.values()
        w = bn.weight / (bn.running_var + bn.eps) ** 0.5
        w = c.weight * w[:, None, None, None]
        b = bn.bias - bn.running_mean * bn.weight / \
            (bn.running_var + bn.eps) ** 0.5
        m = torch.nn.Conv2d(w.size(1) * self.c.groups, w.size(
            0), w.shape[2:], stride=self.c.stride, padding=self.c.padding, dilation=self.c.dilation,
                            groups=self.c.groups,
                            device=c.weight.device)
        m.weight.data.copy_(w)
        m.bias.data.copy_(b)
        return m


class Residual(torch.nn.Module):
    def __init__(self, m, drop=0.):
        super().__init__()
        self.m = m
        self.drop = drop

    def forward(self, x):
        if self.training and self.drop > 0:
            return x + self.m(x) * torch.rand(x.size(0), 1, 1, 1,
                                              device=x.device).ge_(self.drop).div(1 - self.drop).detach()
        else:
            return x + self.m(x)

    @torch.no_grad()
    def fuse(self):
        if isinstance(self.m, Conv2d_BN):
            m = self.m.fuse()
            assert (m.groups == m.in_channels)
            identity = torch.ones(m.weight.shape[0], m.weight.shape[1], 1, 1)
            identity = torch.nn.functional.pad(identity, [1, 1, 1, 1])
            m.weight += identity.to(m.weight.device)
            return m
        elif isinstance(self.m, torch.nn.Conv2d):
            m = self.m
            assert (m.groups != m.in_channels)
            identity = torch.ones(m.weight.shape[0], m.weight.shape[1], 1, 1)
            identity = torch.nn.functional.pad(identity, [1, 1, 1, 1])
            m.weight += identity.to(m.weight.device)
            return m
        else:
            return self


class RepVGGDW(torch.nn.Module):
    def __init__(self, ed) -> None:
        super().__init__()
        self.conv = Conv2d_BN(ed, ed, 3, 1, 1, groups=ed)
        self.conv1 = Conv2d_BN(ed, ed, 1, 1, 0, groups=ed)
        self.dim = ed

    def forward(self, x):
        return self.conv(x) + self.conv1(x) + x

    @torch.no_grad()
    def fuse(self):
        conv = self.conv.fuse()
        conv1 = self.conv1.fuse()

        conv_w = conv.weight
        conv_b = conv.bias
        conv1_w = conv1.weight
        conv1_b = conv1.bias

        conv1_w = torch.nn.functional.pad(conv1_w, [1, 1, 1, 1])

        identity = torch.nn.functional.pad(torch.ones(conv1_w.shape[0], conv1_w.shape[1], 1, 1, device=conv1_w.device),
                                           [1, 1, 1, 1])

        final_conv_w = conv_w + conv1_w + identity
        final_conv_b = conv_b + conv1_b

        conv.weight.data.copy_(final_conv_w)
        conv.bias.data.copy_(final_conv_b)
        return conv


class RepViTBlock(nn.Module):
    def __init__(self, inp, hidden_dim, oup, kernel_size, stride, use_se, use_hs, skip_downsample=False):
        super(RepViTBlock, self).__init__()
        assert stride in [1, 2]

        self.identity = stride == 1 and inp == oup
        assert (hidden_dim == 2 * inp)

        if stride == 2:
            if skip_downsample:
                stride = 1
            self.token_mixer = nn.Sequential(
                Conv2d_BN(inp, inp, kernel_size, stride, (kernel_size - 1) // 2, groups=inp),
                SqueezeExcite(inp, 0.25) if use_se else nn.Identity(),
                Conv2d_BN(inp, oup, ks=1, stride=1, pad=0)
            )
            self.channel_mixer = Residual(nn.Sequential(
                # pw
                Conv2d_BN(oup, 2 * oup, 1, 1, 0),
                nn.GELU() if use_hs else nn.GELU(),
                # pw-linear
                Conv2d_BN(2 * oup, oup, 1, 1, 0, bn_weight_init=0),
            ))
        else:
            assert (self.identity)
            self.token_mixer = nn.Sequential(
                RepVGGDW(inp),
                SqueezeExcite(inp, 0.25) if use_se else nn.Identity(),
            )
            self.channel_mixer = Residual(nn.Sequential(
                # pw
                Conv2d_BN(inp, hidden_dim, 1, 1, 0),
                nn.GELU() if use_hs else nn.GELU(),
                # pw-linear
                Conv2d_BN(hidden_dim, oup, 1, 1, 0, bn_weight_init=0),
            ))

    def forward(self, x):
        return self.channel_mixer(self.token_mixer(x))


from timm.models.vision_transformer import trunc_normal_


class BN_Linear(torch.nn.Sequential):
    def __init__(self, a, b, bias=True, std=0.02):
        super().__init__()
        self.add_module('bn', torch.nn.BatchNorm1d(a))
        self.add_module('l', torch.nn.Linear(a, b, bias=bias))
        trunc_normal_(self.l.weight, std=std)
        if bias:
            torch.nn.init.constant_(self.l.bias, 0)

    @torch.no_grad()
    def fuse(self):
        bn, l = self._modules.values()
        w = bn.weight / (bn.running_var + bn.eps) ** 0.5
        b = bn.bias - self.bn.running_mean * \
            self.bn.weight / (bn.running_var + bn.eps) ** 0.5
        w = l.weight * w[None, :]
        if l.bias is None:
            b = b @ self.l.weight.T
        else:
            b = (l.weight @ b[:, None]).view(-1) + self.l.bias
        m = torch.nn.Linear(w.size(1), w.size(0), device=l.weight.device)
        m.weight.data.copy_(w)
        m.bias.data.copy_(b)
        return m


class RepViT(nn.Module):
    arch_settings = {
        'm1': m1_cfgs,
        'm2': m2_cfgs,
        'm3': m3_cfgs
    }

    def __init__(self, arch, img_size=1024, upsample_mode='bicubic'):
        super(RepViT, self).__init__()
        # setting of inverted residual blocks
        self.cfgs = self.arch_settings[arch]
        self.img_size = img_size

        # building first layer
        input_channel = self.cfgs[0][2]
        patch_embed = torch.nn.Sequential(Conv2d_BN(3, input_channel // 2, 3, 2, 1), torch.nn.GELU(),
                                          Conv2d_BN(input_channel // 2, input_channel, 3, 2, 1))
        layers = [patch_embed]
        # building inverted residual blocks
        block = RepViTBlock
        self.stage_idx = []
        prev_c = input_channel
        for idx, (k, t, c, use_se, use_hs, s) in enumerate(self.cfgs):
            output_channel = _make_divisible(c, 8)
            exp_size = _make_divisible(input_channel * t, 8)
            skip_downsample = False
            if c != prev_c:
                self.stage_idx.append(idx - 1)
                prev_c = c
            layers.append(block(input_channel, exp_size, output_channel, k, s, use_se, use_hs, skip_downsample))
            input_channel = output_channel
        self.stage_idx.append(idx)
        self.features = nn.ModuleList(layers)

        stage2_channels = _make_divisible(self.cfgs[self.stage_idx[2]][2], 8)
        stage3_channels = _make_divisible(self.cfgs[self.stage_idx[3]][2], 8)
        self.fuse_stage2 = nn.Conv2d(stage2_channels, 256, kernel_size=1, bias=False)
        self.fuse_stage3 = OpSequential([
            nn.Conv2d(stage3_channels, 256, kernel_size=1, bias=False),
            UpSampleLayer(factor=2, mode=upsample_mode),
        ])

        self.neck = nn.Sequential(
            nn.Conv2d(256, 256, kernel_size=1, bias=False),
            LayerNorm2d(256),
            nn.Conv2d(256, 256, kernel_size=3, padding=1, bias=False),
            LayerNorm2d(256),
        )

    def forward(self, x):
        counter = 0
        output_dict = dict()
        # patch_embed
        x = self.features[0](x)
        output_dict['stem'] = x
        # stages
        for idx, f in enumerate(self.features[1:]):
            x = f(x)
            if idx in self.stage_idx:
                output_dict[f'stage{counter}'] = x
                counter += 1

        x = self.fuse_stage2(output_dict['stage2']) + self.fuse_stage3(output_dict['stage3'])

        x = self.neck(x)
        return x


def rep_vit_m1(img_size=1024, **kwargs):
    return RepViT('m1', img_size, **kwargs)


def rep_vit_m2(img_size=1024, **kwargs):
    return RepViT('m2', img_size, **kwargs)


def rep_vit_m3(img_size=1024, **kwargs):
    return RepViT('m3', img_size, **kwargs)


# -*- coding: utf-8 -*-
# @Author  : LG

from PyQt5 import QtWidgets, QtCore, QtGui
from PyQt5.QtCore import QThread, pyqtSignal
from ISAT.ui.model_manager_dialog import Ui_Dialog
from ISAT.configs import CHECKPOINT_PATH
from ISAT.segment_any.model_zoo import model_dict
from urllib import request
from functools import partial
import shutil
import time
import os

class DownloadThread(QThread):
    tag = pyqtSignal(float, float)
    def __init__(self, parent=None):
        super(DownloadThread, self).__init__(parent)
        self.urls = None
        self.name = None
        self.pause = False

        self.block_size = 4096

    def setNameAndUrl(self, name, urls):
        self.urls = urls
        self.name = name
        self.pause = False

    def run(self):
        if self.name is not None and self.urls is not None:
            tmp_root = os.path.join(CHECKPOINT_PATH, 'tmp')
            if not os.path.exists(tmp_root):
                os.mkdir(tmp_root)

            # 寻找最佳下载链接
            best_time = 1e8
            best_url = self.urls[0]
            for url in self.urls:
                try:
                    start_time = time.time()
                    req = request.Request(url, headers={"Range": "bytes=0-10"})
                    request.urlopen(req, timeout=5)
                    cost_time = time.time() - start_time
                except:
                    cost_time = 1e8
                if cost_time < best_time:
                    best_time = cost_time
                    best_url = url

            print('Download {} from {}'.format(self.name,  best_url))
            # 检查缓存
            downloaded_size = 0
            download_tmp = os.path.join(tmp_root, self.name)
            if os.path.exists(download_tmp):
                with open(download_tmp, 'rb') as f:
                    downloaded_size = len(f.read())

            req = request.Request(best_url, headers={"Range": "bytes=0-"})
            try:
                response = request.urlopen(req, timeout=10)
                total_size = int(response.headers['Content-Length'])
            except Exception as e:
                print('When download {} from {}, {}'.format(self.name, best_url, e))
                return
            # 存在缓存
            if downloaded_size != 0:
                # 判断缓存文件是否下载完
                if downloaded_size > total_size:
                    self.move_from_tmp(download_tmp, os.path.join(CHECKPOINT_PATH, self.name))
                    return
                # 断点续传
                content_range = response.headers.get('Content-Range', None)
                if content_range is not None:
                    req = request.Request(best_url, headers={"Range": "bytes={}-".format(downloaded_size)})
                    response = request.urlopen(req)
                    content_range = response.headers.get('Content-Range', None)
                    if content_range is not None:
                        print('Resume downloading: ', content_range)
                else:
                    print('Not supprot resume download.')
                    downloaded_size = 0

            open_mode = 'wb' if downloaded_size == 0 else 'ab'
            with open(download_tmp, open_mode) as f:
                while True:
                    if self.pause:
                        self.tag.emit(-1, -1)
                        return
                    buffer = response.read(self.block_size)
                    if not buffer:
                        break
                    f.write(buffer)
                    downloaded_size += len(buffer)
                    self.tag.emit(downloaded_size, total_size)
            # 下载完成
            self.move_from_tmp(download_tmp, os.path.join(CHECKPOINT_PATH, self.name))

    def move_from_tmp(self, from_path, to_path):
        try:
            shutil.move(from_path, to_path)
            self.tag.emit(-2, -2)

        except Exception as e:
            print('Error when move {} to {}, {}'.format(from_path, to_path, e))

class ModelManagerDialog(QtWidgets.QDialog, Ui_Dialog):
    def __init__(self, parent, mainwindow):
        super(ModelManagerDialog, self).__init__(parent)
        self.mainwindow = mainwindow
        self.setupUi(self)
        self.setWindowModality(QtCore.Qt.WindowModality.WindowModal)
        self.download_thread_dict = {}
        self.init_gui()

        self.tableWidget.setColumnWidth(0, 300)
        self.pushButton_clear_tmp.clicked.connect(self.clear_tmp)
        self.checkBox_use_bfloat16.stateChanged.connect(self.use_bfloat16)

    def init_gui(self):
        for index, (name, info_dict) in enumerate(model_dict.items()):
            url = info_dict.get('url', '')
            memory = info_dict.get('memory', '')
            bf16_memory = info_dict.get('bf16_memory', '')
            params = info_dict.get('params', '')
            name_label = QtWidgets.QLabel()
            name_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
            name_label.setText(name)
            # 显存占用
            memory_label = QtWidgets.QLabel()
            memory_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
            memory_label.setText(bf16_memory)
            # 权重大小
            params_label = QtWidgets.QLabel()
            params_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
            params_label.setText(params)
            # 下载/删除按钮
            ops_button = QtWidgets.QPushButton()
            if os.path.exists(os.path.join(CHECKPOINT_PATH, name)):
                # ops_button.setStyleSheet('QWidget {background-color: %s}' % 'red')
                ops_button.setStyleSheet('QWidget {color: %s}' % 'red')
                ops_button.setText('delete')
                ops_button.clicked.connect(self.delete)
            else:
                # ops_button.setStyleSheet('QWidget {background-color: %s}' % 'green')
                ops_button.setStyleSheet('QWidget {color: %s}' % 'green')
                ops_button.setText('download')
                ops_button.clicked.connect(self.download)

            self.tableWidget.insertRow(index)
            self.tableWidget.setCellWidget(index, 0, name_label)
            self.tableWidget.setCellWidget(index, 1, memory_label)
            self.tableWidget.setCellWidget(index, 2, params_label)
            self.tableWidget.setCellWidget(index, 3, ops_button)

    def download(self):
        button = self.sender()
        button.setText('downloading')
        row = self.tableWidget.indexAt(button.pos()).row()
        name_label = self.tableWidget.cellWidget(row, 0)
        name = name_label.text()
        info_dict = model_dict.get(name, None)
        urls = info_dict.get('urls', None)

        if name in self.download_thread_dict:
            download_thread = self.download_thread_dict[name]
        else:
            download_thread = DownloadThread(self)
            self.download_thread_dict[name] = download_thread

        download_thread.setNameAndUrl(name, urls)
        download_thread.tag.connect(partial(self.download_process, button))
        download_thread.start()

        button.setEnabled(False)
        button.clicked.connect(self.pause)
        button.setEnabled(True)

    def download_process(self, button:QtWidgets.QPushButton, downloaded_size, total_size):
        if downloaded_size == -1 and total_size == -1:
            button.setText('continue')
            button.setEnabled(False)
            button.clicked.connect(self.download)
            button.setEnabled(True)

        elif downloaded_size == -2 and total_size == -2:
            button.setText('delete')
            button.setStyleSheet('QWidget {color: %s}' % 'red')
            button.setEnabled(False)
            button.clicked.connect(self.delete)
            button.setEnabled(True)
            self.mainwindow.update_menuSAM()
        else:
            button.setText('{:.2f}% - {}/{}M'.format(downloaded_size/total_size*100, downloaded_size//1000000, total_size//1000000))


    def pause(self):
        button = self.sender()
        row = self.tableWidget.indexAt(button.pos()).row()
        name_label = self.tableWidget.cellWidget(row, 0)
        name = name_label.text()

        download_thread:DownloadThread = self.download_thread_dict[name]
        download_thread.pause = True

    def delete(self):
        button = self.sender()
        row = self.tableWidget.indexAt(button.pos()).row()
        name_label = self.tableWidget.cellWidget(row, 0)
        name = name_label.text()
        try:
            os.remove(os.path.join(CHECKPOINT_PATH, name))
            button.setText('download')
            button.setStyleSheet('QWidget {color: %s}' % 'green')
            button.setEnabled(False)
            button.clicked.connect(self.download)
            button.setEnabled(True)
        except Exception as e:
            print('Error when remove {}, {}'.format(
                os.path.join(CHECKPOINT_PATH, name), e))
        self.mainwindow.update_menuSAM()

    def clear_tmp(self):
        remove_list = []
        for name in os.listdir(os.path.join(CHECKPOINT_PATH, 'tmp')):
            tmp_file = os.path.join(CHECKPOINT_PATH, 'tmp', name)
            if name in self.download_thread_dict:
                thread = self.download_thread_dict.get(name, None)
                if thread is not None:
                    if thread.isRunning():
                        continue
            try:
                os.remove(tmp_file)
                remove_list.append(tmp_file)

            except Exception as e:
                print('Error when remove {}, {}'.format(
                    os.path.join(CHECKPOINT_PATH, name), e))
        QtWidgets.QMessageBox.information(
            self,
            'clear tmp',
            'Remove tmp: [' + '],['.join(remove_list) + ']'
        )

    def update_gui(self):
        self.checkBox_use_bfloat16.setChecked(self.mainwindow.cfg['software']['use_bfloat16'])

        for index, (name, info_dict) in enumerate(model_dict.items()):
            memory = info_dict.get('memory', '')
            bf16_memory = info_dict.get('bf16_memory', '')
            # 显存占用
            memory_label = QtWidgets.QLabel()
            memory_label.setAlignment(QtCore.Qt.AlignmentFlag.AlignCenter)
            if self.mainwindow.cfg['software']['use_bfloat16']:
                memory_label.setText(bf16_memory)
            else:
                memory_label.setText(memory)
            self.tableWidget.setCellWidget(index, 1, memory_label)

    def use_bfloat16(self):
        use = self.checkBox_use_bfloat16.isChecked()
        self.mainwindow.change_bfloat16_state(use)